Attribute VB_Name = "Module1"
Option Explicit

Public Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

'Function OpenThePort() As Integer


'About overlapped I/O
'
'Reading HID Input reports is done with the API function ReadFile.
'Non-overlapped ReadFile is a blocking call. If the device doesn't return the
'expected amount of data, the application hangs and waits endlessly.
'application
'With overlapped I/O, the call to ReadFile returns immediately. The  uses
'WaitForSingleObject to be notified either that the data has arrived or that a timeout
'has occurred.
'WaitForSingleObject blocks the application thread but specifies a timeout value so the
'application's thread isn't blocked forever.
'
'This application has been tested on Windows 98 SE and Windows 2000.

Dim bAlertable As Long
Dim Capabilities As HIDP_CAPS
Dim DataString As String
Dim DetailData As Long
Dim DetailDataBuffer() As Byte
Dim DeviceAttributes As HIDD_ATTRIBUTES
Dim DevicePathName As String
Dim DeviceInfoSet As Long
Dim ErrorString As String
Dim EventObject As Long
Dim HIDHandle As Long
Dim HIDOverlapped As OVERLAPPED
Dim LastDevice As Boolean
Dim MyDeviceDetected As Boolean
Dim MyDeviceInfoData As SP_DEVINFO_DATA
Dim MyDeviceInterfaceDetailData As SP_DEVICE_INTERFACE_DETAIL_DATA
Dim MyDeviceInterfaceData As SP_DEVICE_INTERFACE_DATA
Dim Needed As Long
Dim PreparsedData As Long
Public ReadBuffer() As Byte

Dim ReadHandle As Long
Dim Result As Long

'Set these to match the values in the device's firmware and INF file.
'0925h is Lakeview Research's vendor ID.
Const MyVendorID = 1240
Const MyProductID = 50

Const INCADDRESS = &H49 ' I
Const ENTERPGM = &H50   ' P - enter program mode
Const EXITPGM = &H70    ' p - exit program mode
Const WRITEPGM = &H57   ' W
Const WRITECONFIG = &H43 ' C - Set address to config memory
Const WRITEDATA = &H44  ' D
Const ERASEPGM = &H45   ' E
Const ERASEDATA = &H65  ' e
Const READPGM = &H52    ' R
Const READDATA = &H72   ' r
Const CHKSUM = &H53     ' S
Const GETVERSION = &H76 ' v
Const POWERCTRL = &H56  ' V
Const NULLCMD = 122     ' Z
Const CMDTABLE = &H63   ' c - Download 6 bit programming command table

Const MAXDATASAMPLES = 87000
Global Temperature(MAXDATASAMPLES) As Single
Global Photo(MAXDATASAMPLES) As Single
Global Humidity(MAXDATASAMPLES) As Single

Function FindTheHid() As Boolean
'Makes a series of API calls to locate the desired HID-class device.
'Returns True if the device is detected, False if not detected.

Dim Count As Integer
Dim GUIDString As String
Dim HidGuid As GUID
Dim MemberIndex As Long

LastDevice = False
MyDeviceDetected = False

'******************************************************************************
'HidD_GetHidGuid
'Get the GUID for all system HIDs.
'Returns: the GUID in HidGuid.
'The routine doesn't return a value in Result
'but the routine is declared as a function for consistency with the other API calls.
'******************************************************************************

Result = HidD_GetHidGuid(HidGuid)
Call DisplayResultOfAPICall("GetHidGuid")

'Display the GUID.
GUIDString = _
    Hex$(HidGuid.Data1) & "-" & _
    Hex$(HidGuid.Data2) & "-" & _
    Hex$(HidGuid.Data3) & "-"

For Count = 0 To 7
    'Ensure that each of the 8 bytes in the GUID displays two characters.
    If HidGuid.Data4(Count) >= &H10 Then
        GUIDString = GUIDString & Hex$(HidGuid.Data4(Count)) & " "
    Else
        GUIDString = GUIDString & "0" & Hex$(HidGuid.Data4(Count)) & " "
    End If
Next Count

'lstResults.AddItem "  GUID for system HIDs: " & GUIDString

'******************************************************************************
'SetupDiGetClassDevs
'Returns: a handle to a device information set for all installed devices.
'Requires: the HidGuid returned in GetHidGuid.
'******************************************************************************

DeviceInfoSet = SetupDiGetClassDevs _
    (HidGuid, _
    vbNullString, _
    0, _
    (DIGCF_PRESENT Or DIGCF_DEVICEINTERFACE))
    
Call DisplayResultOfAPICall("SetupDiClassDevs")
DataString = GetDataString(DeviceInfoSet, 32)

'******************************************************************************
'SetupDiEnumDeviceInterfaces
'On return, MyDeviceInterfaceData contains the handle to a
'SP_DEVICE_INTERFACE_DATA structure for a detected device.
'Requires:
'the DeviceInfoSet returned in SetupDiGetClassDevs.
'the HidGuid returned in GetHidGuid.
'An index to specify a device.
'******************************************************************************

'Begin with 0 and increment until no more devices are detected.
MemberIndex = 0

Do
    'The cbSize element of the MyDeviceInterfaceData structure must be set to
    'the structure's size in bytes. The size is 28 bytes.
    MyDeviceInterfaceData.cbSize = LenB(MyDeviceInterfaceData)
    Result = SetupDiEnumDeviceInterfaces _
        (DeviceInfoSet, _
        0, _
        HidGuid, _
        MemberIndex, _
        MyDeviceInterfaceData)
    
    Call DisplayResultOfAPICall("SetupDiEnumDeviceInterfaces")
    If Result = 0 Then LastDevice = True
    
    'If a device exists, display the information returned.
    If Result <> 0 Then
'        lstResults.AddItem "  DeviceInfoSet for device #" & CStr(MemberIndex) & ": "
'        lstResults.AddItem "  cbSize = " & CStr(MyDeviceInterfaceData.cbSize)
'        lstResults.AddItem _
            "  InterfaceClassGuid.Data1 = " & Hex$(MyDeviceInterfaceData.InterfaceClassGuid.Data1)
'        lstResults.AddItem _
            "  InterfaceClassGuid.Data2 = " & Hex$(MyDeviceInterfaceData.InterfaceClassGuid.Data2)
'        lstResults.AddItem _
            "  InterfaceClassGuid.Data3 = " & Hex$(MyDeviceInterfaceData.InterfaceClassGuid.Data3)
'        lstResults.AddItem _
            "  Flags = " & Hex$(MyDeviceInterfaceData.Flags)
    
        
        '******************************************************************************
        'SetupDiGetDeviceInterfaceDetail
        'Returns: an SP_DEVICE_INTERFACE_DETAIL_DATA structure
        'containing information about a device.
        'To retrieve the information, call this function twice.
        'The first time returns the size of the structure in Needed.
        'The second time returns a pointer to the data in DeviceInfoSet.
        'Requires:
        'A DeviceInfoSet returned by SetupDiGetClassDevs and
        'an SP_DEVICE_INTERFACE_DATA structure returned by SetupDiEnumDeviceInterfaces.
        '*******************************************************************************
        
        MyDeviceInfoData.cbSize = Len(MyDeviceInfoData)
        Result = SetupDiGetDeviceInterfaceDetail _
           (DeviceInfoSet, _
           MyDeviceInterfaceData, _
           0, _
           0, _
           Needed, _
           0)
        
        DetailData = Needed
            
        Call DisplayResultOfAPICall("SetupDiGetDeviceInterfaceDetail")
   '     lstResults.AddItem "  (OK to say too small)"
   '     lstResults.AddItem "  Required buffer size for the data: " & Needed
        
        'Store the structure's size.
        MyDeviceInterfaceDetailData.cbSize = _
            Len(MyDeviceInterfaceDetailData)
        
        'Use a byte array to allocate memory for
        'the MyDeviceInterfaceDetailData structure
        ReDim DetailDataBuffer(Needed)
        'Store cbSize in the first four bytes of the array.
        Call RtlMoveMemory _
            (DetailDataBuffer(0), _
            MyDeviceInterfaceDetailData, _
            4)
        
        'Call SetupDiGetDeviceInterfaceDetail again.
        'This time, pass the address of the first element of DetailDataBuffer
        'and the returned required buffer size in DetailData.
        Result = SetupDiGetDeviceInterfaceDetail _
           (DeviceInfoSet, _
           MyDeviceInterfaceData, _
           VarPtr(DetailDataBuffer(0)), _
           DetailData, _
           Needed, _
           0)
        
        Call DisplayResultOfAPICall(" Result of second call: ")
   '     lstResults.AddItem "  MyDeviceInterfaceDetailData.cbSize: " & _
            CStr(MyDeviceInterfaceDetailData.cbSize)
        
        'Convert the byte array to a string.
        DevicePathName = CStr(DetailDataBuffer())
        'Convert to Unicode.
        DevicePathName = StrConv(DevicePathName, vbUnicode)
        'Strip cbSize (4 bytes) from the beginning.
        DevicePathName = Right$(DevicePathName, Len(DevicePathName) - 4)
   '     lstResults.AddItem "  Device pathname: "
   '     lstResults.AddItem "    " & DevicePathName
                
        '******************************************************************************
        'CreateFile
        'Returns: a handle that enables reading and writing to the device.
        'Requires:
        'The DevicePathName returned by SetupDiGetDeviceInterfaceDetail.
        '******************************************************************************
    
        HIDHandle = CreateFile _
            (DevicePathName, _
            GENERIC_READ Or GENERIC_WRITE, _
            (FILE_SHARE_READ Or FILE_SHARE_WRITE), _
            0, _
            OPEN_EXISTING, _
            0&, _
            0)
            
        Call DisplayResultOfAPICall("CreateFile")
    '    lstResults.AddItem "  Returned handle: " & Hex$(HIDHandle) & "h"
        
        'Now we can find out if it's the device we're looking for.
        
        '******************************************************************************
        'HidD_GetAttributes
        'Requests information from the device.
        'Requires: The handle returned by CreateFile.
        'Returns: an HIDD_ATTRIBUTES structure containing
        'the Vendor ID, Product ID, and Product Version Number.
        'Use this information to determine if the detected device
        'is the one we're looking for.
        '******************************************************************************
        
        'Set the Size property to the number of bytes in the structure.
        DeviceAttributes.Size = LenB(DeviceAttributes)
        Result = HidD_GetAttributes _
            (HIDHandle, _
            DeviceAttributes)
            
        Call DisplayResultOfAPICall("HidD_GetAttributes")
 '       If Result <> 0 Then
 '           lstResults.AddItem "  HIDD_ATTRIBUTES structure filled without error."
 '       Else
 '           lstResults.AddItem "  Error in filling HIDD_ATTRIBUTES structure."
 '       End If
    
'        lstResults.AddItem "  Structure size: " & DeviceAttributes.Size
'        lstResults.AddItem "  Vendor ID: " & Hex$(DeviceAttributes.VendorID)
'        lstResults.AddItem "  Product ID: " & Hex$(DeviceAttributes.ProductID)
 '       lstResults.AddItem "  Version Number: " & Hex$(DeviceAttributes.VersionNumber)
        
        'Find out if the device matches the one we're looking for.
        If (DeviceAttributes.VendorID = MyVendorID) And _
            (DeviceAttributes.ProductID = MyProductID) Then
                'It's the desired device.
 '               lstResults.AddItem "  My device detected"
                MyDeviceDetected = True
        Else
                MyDeviceDetected = False
                'If it's not the one we want, close its handle.
                Result = CloseHandle _
                    (HIDHandle)
                DisplayResultOfAPICall ("CloseHandle")
        End If
End If
    'Keep looking until we find the device or there are no more left to examine.
    MemberIndex = MemberIndex + 1
Loop Until (LastDevice = True) Or (MyDeviceDetected = True)

'Free the memory reserved for the DeviceInfoSet returned by SetupDiGetClassDevs.
Result = SetupDiDestroyDeviceInfoList _
    (DeviceInfoSet)
Call DisplayResultOfAPICall("DestroyDeviceInfoList")

If MyDeviceDetected = True Then
    FindTheHid = True
    'Learn the capabilities of the device
     Call GetDeviceCapabilities
    'Get another handle for the overlapped ReadFiles.
    ReadHandle = CreateFile _
            (DevicePathName, _
            (GENERIC_READ Or GENERIC_WRITE), _
            (FILE_SHARE_READ Or FILE_SHARE_WRITE), _
            0&, _
            OPEN_EXISTING, _
            FILE_FLAG_OVERLAPPED, _
            0)

    Call DisplayResultOfAPICall("CreateFile, ReadHandle")
    Call PrepareForOverlappedTransfer
End If

End Function

Private Function GetDataString _
    (Address As Long, _
    Bytes As Long) _
As String

'Retrieves a string of length Bytes from memory, beginning at Address.
'Adapted from Dan Appleman's "Win32 API Puzzle Book"

Dim Offset As Integer
Dim Result$
Dim ThisByte As Byte

For Offset = 0 To Bytes - 1
    Call RtlMoveMemory(ByVal VarPtr(ThisByte), ByVal Address + Offset, 1)
    If (ThisByte And &HF0) = 0 Then
        Result$ = Result$ & "0"
    End If
    Result$ = Result$ & Hex$(ThisByte) & " "
Next Offset

GetDataString = Result$
End Function

Private Function GetErrorString _
    (ByVal LastError As Long) _
As String

'Returns the error message for the last error.
'Adapted from Dan Appleman's "Win32 API Puzzle Book"

Dim Bytes As Long
Dim ErrorString As String
ErrorString = String$(129, 0)
Bytes = FormatMessage _
    (FORMAT_MESSAGE_FROM_SYSTEM, _
    0&, _
    LastError, _
    0, _
    ErrorString$, _
    128, _
    0)
    
'Subtract two characters from the message to strip the CR and LF.
If Bytes > 2 Then
    GetErrorString = Left$(ErrorString, Bytes - 2)
End If

End Function

Private Sub DisplayResultOfAPICall(FunctionName As String)

'Display the results of an API call.

Dim ErrorString As String

'lstResults.AddItem ""
ErrorString = GetErrorString(Err.LastDllError)
'lstResults.AddItem FunctionName
'lstResults.AddItem "  Result = " & ErrorString

'Scroll to the bottom of the list box.
'lstResults.ListIndex = lstResults.ListCount - 1

End Sub


Private Sub GetDeviceCapabilities()

'******************************************************************************
'HidD_GetPreparsedData
'Returns: a pointer to a buffer containing information about the device's capabilities.
'Requires: A handle returned by CreateFile.
'There's no need to access the buffer directly,
'but HidP_GetCaps and other API functions require a pointer to the buffer.
'******************************************************************************

Dim ppData(29) As Byte
Dim ppDataString As Variant

'Preparsed Data is a pointer to a routine-allocated buffer.
Result = HidD_GetPreparsedData _
    (HIDHandle, _
    PreparsedData)
Call DisplayResultOfAPICall("HidD_GetPreparsedData")

'Copy the data at PreparsedData into a byte array.

Result = RtlMoveMemory _
    (ppData(0), _
    PreparsedData, _
    30)
Call DisplayResultOfAPICall("RtlMoveMemory")

ppDataString = ppData()
'Convert the data to Unicode.
ppDataString = StrConv(ppDataString, vbUnicode)

'******************************************************************************
'HidP_GetCaps
'Find out the device's capabilities.
'For standard devices such as joysticks, you can find out the specific
'capabilities of the device.
'For a custom device, the software will probably know what the device is capable of,
'so this call only verifies the information.
'Requires: The pointer to a buffer containing the information.
'The pointer is returned by HidD_GetPreparsedData.
'Returns: a Capabilites structure containing the information.
'******************************************************************************
Result = HidP_GetCaps _
    (PreparsedData, _
    Capabilities)

Call DisplayResultOfAPICall("HidP_GetCaps")
'lstResults.AddItem "  Last error: " & ErrorString
'lstResults.AddItem "  Usage: " & Hex$(Capabilities.Usage)
'lstResults.AddItem "  Usage Page: " & Hex$(Capabilities.UsagePage)
'lstResults.AddItem "  Input Report Byte Length: " & Capabilities.InputReportByteLength
'lstResults.AddItem "  Output Report Byte Length: " & Capabilities.OutputReportByteLength
'lstResults.AddItem "  Feature Report Byte Length: " & Capabilities.FeatureReportByteLength
'lstResults.AddItem "  Number of Link Collection Nodes: " & Capabilities.NumberLinkCollectionNodes
'lstResults.AddItem "  Number of Input Button Caps: " & Capabilities.NumberInputButtonCaps
'lstResults.AddItem "  Number of Input Value Caps: " & Capabilities.NumberInputValueCaps
'lstResults.AddItem "  Number of Input Data Indices: " & Capabilities.NumberInputDataIndices
'lstResults.AddItem "  Number of Output Button Caps: " & Capabilities.NumberOutputButtonCaps
'lstResults.AddItem "  Number of Output Value Caps: " & Capabilities.NumberOutputValueCaps
'lstResults.AddItem "  Number of Output Data Indices: " & Capabilities.NumberOutputDataIndices
'lstResults.AddItem "  Number of Feature Button Caps: " & Capabilities.NumberFeatureButtonCaps
'lstResults.AddItem "  Number of Feature Value Caps: " & Capabilities.NumberFeatureValueCaps
'lstResults.AddItem "  Number of Feature Data Indices: " & Capabilities.NumberFeatureDataIndices

'******************************************************************************
'HidP_GetValueCaps
'Returns a buffer containing an array of HidP_ValueCaps structures.
'Each structure defines the capabilities of one value.
'This application doesn't use this data.
'******************************************************************************

'This is a guess. The byte array holds the structures.
Dim ValueCaps(1023) As Byte

Result = HidP_GetValueCaps _
    (HidP_Input, _
    ValueCaps(0), _
    Capabilities.NumberInputValueCaps, _
    PreparsedData)
   
Call DisplayResultOfAPICall("HidP_GetValueCaps")

'lstResults.AddItem "ValueCaps= " & GetDataString((VarPtr(ValueCaps(0))), 180)
'To use this data, copy the byte array into an array of structures.

'Free the buffer reserved by HidD_GetPreparsedData
Result = HidD_FreePreparsedData _
    (PreparsedData)
Call DisplayResultOfAPICall("HidD_FreePreparsedData")

End Sub


Private Sub PrepareForOverlappedTransfer()

'******************************************************************************
'CreateEvent
'Creates an event object for the overlapped structure used with ReadFile.
'Requires a security attributes structure or null,
'Manual Reset = True (ResetEvent resets the manual reset object to nonsignaled),
'Initial state = True (signaled),
'and event object name (optional)
'Returns a handle to the event object.
'******************************************************************************

If EventObject = 0 Then
    EventObject = CreateEvent _
        (0&, _
        True, _
        True, _
        "")
End If
    
Call DisplayResultOfAPICall("CreateEvent")
    
'Set the members of the overlapped structure.
HIDOverlapped.Offset = 0
HIDOverlapped.OffsetHigh = 0
HIDOverlapped.hEvent = EventObject
End Sub


Public Sub WriteToDevice(b1 As Byte, b2 As Byte, b3 As Byte, b4 As Byte, b5 As Byte, b6 As Byte, b7 As Byte, b8 As Byte)
Dim Count As Integer
Dim NumberOfBytesRead As Long
Dim NumberOfBytesToSend As Long
Dim NumberOfBytesWritten As Long
Dim SendBuffer() As Byte


'If the device hasn't been detected or it timed out on a previous attempt
'to access it, look for the device.
If MyDeviceDetected = False Then
    MyDeviceDetected = FindTheHid
End If

If MyDeviceDetected = True Then

'The SendBuffer array begins at 0, so subtract 1 from the number of bytes.
    ReDim SendBuffer(Capabilities.OutputReportByteLength - 1)

'******************************************************************************
'WriteFile
'Sends a report to the device.
'Returns: success or failure.
'Requires: the handle returned by CreateFile and
'The output report byte length returned by HidP_GetCaps
'******************************************************************************

    'The first byte is the Report ID
    SendBuffer(0) = 0
    SendBuffer(1) = b1
    SendBuffer(2) = b2
    SendBuffer(3) = b3
    SendBuffer(4) = b4
    SendBuffer(5) = b5
    SendBuffer(6) = b6
    SendBuffer(7) = b7
    SendBuffer(8) = b8


    NumberOfBytesWritten = 0

    Result = WriteFile _
        (HIDHandle, _
        SendBuffer(0), _
         CLng(Capabilities.OutputReportByteLength), _
        NumberOfBytesWritten, _
        0)

    Call DisplayResultOfAPICall("WriteFile")

    ' Read a report from the device.
    ' Call ReadReport
Else
End If

End Sub

Public Function ReadReport() As Long

'Read data from the device.

Dim Count
Dim NumberOfBytesRead As Long
'Allocate a buffer for the report.
'Byte 0 is the report ID.
Dim UBoundReadBuffer As Integer

'******************************************************************************
'ReadFile
'Returns: the report in ReadBuffer.
'Requires: a device handle returned by CreateFile
'(for overlapped I/O, CreateFile must be called with FILE_FLAG_OVERLAPPED),
'the Input report length in bytes returned by HidP_GetCaps,
'and an overlapped structure whose hEvent member is set to an event object.
'******************************************************************************

Dim ReadResult As Long

Dim ByteValue As String
'The ReadBuffer array begins at 0, so subtract 1 from the number of bytes.
'ReDim ReadBuffer(Capabilities.InputReportByteLength - 1)
ReDim ReadBuffer(8)
'Scroll to the bottom of the list box.
'lstResults.ListIndex = lstResults.ListCount - 1

'Do an overlapped ReadFile.
'The function returns immediately, even if the data hasn't been received yet.
Result = ReadFile _
    (ReadHandle, _
    ReadBuffer(0), _
    CLng(Capabilities.InputReportByteLength), _
    NumberOfBytesRead, _
    HIDOverlapped)
ReadReport = Result
'Call DisplayResultOfAPICall("ReadFile")

'lstResults.AddItem "waiting for ReadFile"
'Scroll to the bottom of the list box.
'lstResults.ListIndex = lstResults.ListCount - 1
bAlertable = True

'******************************************************************************
'WaitForSingleObject
'Used with overlapped ReadFile.
'Returns when ReadFile has received the requested amount of data or on timeout.
'Requires an event object created with CreateEvent
'and a timeout value in milliseconds.
'******************************************************************************
Result = WaitForSingleObject _
    (EventObject, _
    1000)
Call DisplayResultOfAPICall("WaitForSingleObject")

'Find out if ReadFile completed or timeout.
Select Case Result
    Case WAIT_OBJECT_0
        'ReadFile has completed
'        lstResults.AddItem "ReadFile completed successfully."
    Case WAIT_TIMEOUT
        'Timeout
 '       lstResults.AddItem "Readfile timeout"
        'Cancel the operation
        
        '*************************************************************
        'CancelIo
        'Cancels the ReadFile
        'Requires the device handle.
        'Returns non-zero on success.
        '*************************************************************
        Result = CancelIo _
            (ReadHandle)
'        lstResults.AddItem "************ReadFile timeout*************"
'        lstResults.AddItem "CancelIO"
        Call DisplayResultOfAPICall("CancelIo")
        'The timeout may have been because the device was removed,
        'so close any open handles and
        'set MyDeviceDetected=False to cause the application to
        'look for the device on the next attempt.
        CloseHandle (HIDHandle)
        Call DisplayResultOfAPICall("CloseHandle (HIDHandle)")
        CloseHandle (ReadHandle)
        Call DisplayResultOfAPICall("CloseHandle (ReadHandle)")
        MyDeviceDetected = False
    Case Else
'        lstResults.AddItem "Readfile undefined error"
        MyDeviceDetected = False
End Select
    
'lstResults.AddItem " Report ID: " & ReadBuffer(0)
'lstResults.AddItem " Report Data:"


'******************************************************************************
'ResetEvent
'Sets the event object in the overlapped structure to non-signaled.
'Requires a handle to the event object.
'Returns non-zero on success.
'******************************************************************************

Call ResetEvent(EventObject)
Call DisplayResultOfAPICall("ResetEvent")
End Function


Public Sub Shutdown()
'Actions that must execute when the program ends.

'Close the open handles to the device.
Result = CloseHandle _
    (HIDHandle)
'Call DisplayResultOfAPICall("CloseHandle (HIDHandle)")

Result = CloseHandle _
    (ReadHandle)
'Call DisplayResultOfAPICall("CloseHandle (ReadHandle)")

End Sub






Function Dec2Bin(MyByte As Byte) As String
    Dim CurrentData As Integer
    Dim OldData As Integer
    Dim i As Integer
    
    Dec2Bin = ""
    OldData = MyByte
    
    For i = 7 To 0 Step -1
     
        CurrentData = OldData - (2 ^ i)
        If CurrentData < 0 Then
            Dec2Bin = Dec2Bin & "0"
        Else
            OldData = CurrentData
            Dec2Bin = Dec2Bin & "1"
        End If
    
    Next i
    
End Function
Function Dec2BinLong(MyInt As Integer) As String
    Dim CurrentData As Integer
    Dim OldData As Integer
    Dim i As Integer
    
    Dec2BinLong = ""
    OldData = MyInt
    
    For i = 15 To 0 Step -1
     
        CurrentData = OldData - (2 ^ i)
        If CurrentData < 0 Then
            Dec2BinLong = Dec2BinLong & "0"
        Else
            OldData = CurrentData
            Dec2BinLong = Dec2BinLong & "1"
        End If
    
    Next i
    
End Function


Function Dec2Hex(MyInteger As Integer, MyWidth As Integer) As String
    Dim TempWork As String
    
    TempWork = Hex(MyInteger)
    
    If Len(TempWork) > CLng(MyWidth) Then
        Dec2Hex = Mid(TempWork, Len(TempWork) - MyWidth, MyWidth)
        Exit Function
    End If
    
    Do Until Len(TempWork) = CLng(MyWidth)
        TempWork = "0" & TempWork
    Loop
    
    Dec2Hex = TempWork
End Function

Function LongDec2Hex(MyLong As Long, MyWidth As Integer) As String
    Dim TempWork As String
    
    TempWork = Hex(MyLong)
    
    If Len(TempWork) > CLng(MyWidth) Then
        LongDec2Hex = Mid(TempWork, Len(TempWork) - MyWidth, MyWidth)
        Exit Function
    End If
    
    Do Until Len(TempWork) = CLng(MyWidth)
        TempWork = "0" & TempWork
    Loop
    
    LongDec2Hex = TempWork
End Function


Function Unsign2Sign(MyByte As Byte) As Integer
    Dim TempData As Integer
    
    If (MyByte And 128) > 0 Then
        Unsign2Sign = CInt(MyByte - 256)
    Else
        Unsign2Sign = MyByte
    End If
End Function

' Function CheckBoard looks to see if indeed the PICkit 1 board is attached
' and if there is a device in the socket.
'Public Function CheckBoard() As Boolean
'    MessageBox.BackColor = &H8000000F
'    MessageBox.Caption = "Checking Device..."
'    DoEvents
'    ProgrammerVersion = ReadTheVersion
'
'    If ProgrammerVersion = "0.0.0" Then
'        ProgrammerVersion = "Programmer not found..."
'        MDIForm1.lblMessage.Caption = ProgrammerVersion
'        CheckBoard = False
'    Else
'        If (ProgrammerVersion = "1.0.0") Then
'        ' Old firmware version - disable unsupported functions
'            MDIForm1.cboSpeed.Enabled = False
'            MDIForm1.cboSamples.Enabled = False
'            MDIForm1.cboTrigger.Enabled = False
'            MDIForm1.tbTrigger.Enabled = False
'            MDIForm1.cmdFFT.Enabled = False
'            MDIForm1.cmdOScope.Enabled = False
'            MDIForm1.cmdHistogram.Enabled = False
'            MDIForm1.cmdStripChart.Enabled = False
'            MDIForm1.cmdGo.Enabled = False
'            MDIForm1.cmdStop.Enabled = False
'            MDIForm1.Label3.Enabled = False
'            MDIForm1.Label4.Enabled = False
'        Else
'            InquireDevice
'            If (DeviceName = "Not Present") Then
'                MDIForm1.lblMessage.Caption = "Insert Device"
'                PowerDownDevice
'                CheckBoard = False
'            Else ' We have new firmware and a device is present.
'                CheckBoard = True
'                Select Case DeviceName  ' download the appropriate command table
'                Case "PIC12F629", "PIC12F675", "PIC16F630", "PIC16F676"
'                    Call WriteToDevice(CMDTABLE, &H0, &H2, &H3, &H4, &H5, &H6, &H8)
'                    Call WriteToDevice(&H18, &HA, &H9, &HB, &HFF, &HFF, &HFF, &HFF)
'                Case "PIC16F716"
'                    Call WriteToDevice(CMDTABLE, &H0, &H2, &H3, &H4, &H5, &H6, &H8)
'                    Call WriteToDevice(&H18, &HE, &H9, &HB, &HFF, &HFF, &HFF, &HFF)
'                Case "PIC16F27A", "PIC16F28A", "PIC16F48A"
'                    Call WriteToDevice(CMDTABLE, &H0, &H2, &H3, &H4, &H5, &H6, &H8)
'                    Call WriteToDevice(&H18, &HA, &H9, &HB, &HFF, &HFF, &HFF, &HFF)
'                Case "PIC16F635", "PIC16F636", "PIC12F683", "PIC16F684", "PIC16F688"
'                    Call WriteToDevice(CMDTABLE, &H0, &H2, &H3, &H4, &H5, &H6, &H8)
'                    Call WriteToDevice(&H18, &HA, &H9, &HB, &HFF, &HFF, &HFF, &HFF)
'                End Select
'
'            End If
'        End If
'
'    End If
'End Function
'
' Gets the firmware version of the PICkit board
' tries three times before giving up.
Function ReadTheVersion() As String
    Dim VerString As String
    Dim i As Integer

    For i = 1 To 3
        Call WriteToDevice(GETVERSION, NULLCMD, NULLCMD, NULLCMD, NULLCMD, NULLCMD, NULLCMD, NULLCMD)
        ReadReport
        VerString = Chr$(ReadBuffer(1) + Asc("0")) & "." & Chr$(ReadBuffer(2) + Asc("0")) & "." & Chr$(ReadBuffer(3) + Asc("0"))
        If ReadBuffer(1) <> 0 Then Exit For
    Next i
    ReadTheVersion = VerString
End Function
' Gets the Device Id from the device on the board, looks it up and displays
' the device name on the dialog box
'Public Sub InquireDevice()
'    Dim CSum As Long
'    Dim MaskedConfig As Integer
'    Dim BufferCSum As Long
'
'    PowerDownDevice
'    Sleep 500
'
'    Call WriteToDevice(CMDTABLE, &H0, &H2, &H3, &H4, &H5, &H6, &H8)
'    Call WriteToDevice(&H18, &HA, &H9, &HB, &HFF, &HFF, &HFF, &HFF)
'    Call WriteToDevice(ENTERPGM, WRITECONFIG, READPGM, NULLCMD, NULLCMD, NULLCMD, NULLCMD, NULLCMD) '
'    ReadReport
'
'    Call WriteToDevice(READPGM, EXITPGM, NULLCMD, NULLCMD, NULLCMD, NULLCMD, NULLCMD, NULLCMD) '
'    ReadReport
'    DevId = ReadBuffer(5) + ReadBuffer(6) * 256
'    LookupDevice
'    ProgCon.TBDeviceID = DeviceName
'
'    If (DeviceSaveBGBits = True) Then
'        BandGap = (ReadBuffer(8) * 256) And &H3000
'        ProgCon.lblBandGap = "0x" & Dec2Hex(CInt(BandGap), 4)
'    End If
'
'    If (DeviceSaveOscCal = True) Then
'        Call WriteToDevice(ENTERPGM, INCADDRESS, &HFF, 3, READPGM, EXITPGM, NULLCMD, NULLCMD)
'        Sleep (500) ' wait for it to count up
'        ReadReport
'        OscCalL = ReadBuffer(1)
'        OscCalH = ReadBuffer(2)
'        ProgCon.lblOscCal = "0x" & Dec2Hex(CInt(OscCalH), 2) & Dec2Hex(CInt(OscCalL), 2)
'    End If
'    SendPowerControl
'End Sub


Public Sub PowerUpDevice()
    Call WriteToDevice(POWERCTRL, 1, NULLCMD, NULLCMD, NULLCMD, NULLCMD, NULLCMD, NULLCMD)
End Sub

' Powers down the device and the 2.5kHz oscillator.
Public Sub PowerDownDevice()
    Call WriteToDevice(POWERCTRL, 0, NULLCMD, NULLCMD, NULLCMD, NULLCMD, NULLCMD, NULLCMD)
End Sub

