;----------------------------------------------------------------------
;
;   Filename:           initialize_f684.asm
;   Microcontroller:    PIC16F684
;   Assembled using:    MPLAB IDE v6.40
;
;   Author:             Steven Bible
;   Company:            Microchip Technology Inc.
;
;----------------------------------------------------------------------
;
;   Program Description:
;
;   < type program description here >
;
;----------------------------------------------------------------------
;
;   Revision History:
;
;   Ver. 0.0 - Feburary 25, 2004: Initial write
;
;----------------------------------------------------------------------
; Include Files and Project Files:
;

#include    "project.inc"           ; io port and processor defines

    errorlevel  -302                ; suppress message 302 from list file

;----------------------------------------------------------------------
; Externally Defined Labels
;
; EXTERN declares symbol names that are used in this module and declared
; as global in another module.
;----------------------------------------------------------------------

    ; external variables

    

    ; external subroutines

    

;----------------------------------------------------------------------
; Global Defines
;----------------------------------------------------------------------

; see project.inc

;----------------------------------------------------------------------
; Local Defines
;----------------------------------------------------------------------

; none

;----------------------------------------------------------------------
; Variables
;----------------------------------------------------------------------

;--------------------
; Uninitialized Data Section
;
; The RES directive should be used to reserve space for data storage.

;       udata

;foo     res 1

;--------------------
; Shared Uninitialized Data Section
;
; Variables that are allocated in RAM that are shared across all RAM banks (unbanked RAM). 
; The RES directive should be used to reserve space for data storage.
;
;        udata_shr

;bar     res 1


;--------------------
; Global Variables
;
; Variables defined in this module that should be available to other
; modules. The <label> must be defined prior to using the GLOBAL directive. 

;       global  <label>


;----------------------------------------------------------------------
; Begin Executable Code Segment
;----------------------------------------------------------------------

;[<label>]   code    [<ROM_Address>]

        code

;----------------------------------------------------------------------
; Subroutine: Initialize PICmicro
;

initialize_f684
        global  initialize_f684

;----------------------------------------
; Clock Sources (Section 3.0)
;
; The PIC16F684 has a wide variety of clock sources and selection features
; to allow it to be used in a wide range of applications while maximizing 
; performance, power consumption, and cost.

        ; Oscillator Control Register (OSCCON)

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        ; Internal Oscililator Frequency Select: 110 = 4 MHz
        bsf     OSCCON, IRCF2       ; Internal Oscillator Frequency Select bit 2
        bsf     OSCCON, IRCF1       ; Internal Oscillator Frequency Select bit 1
        bcf     OSCCON, IRCF0       ; Internal Oscillator Frequency Select bit 0

        bsf     OSCCON, SCS         ; System Clock Select: Internal Oscillator

        ; Oscillator Tuning Register: 00000 = Center Frequency

        movlw   0x00
        movwf   OSCTUNE

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; PORTS A AND C (Section 4.0)
;
; Store PORTATRIS and PORTCTRIS values defined in "project.inc" into 
; the TRISA and TRISC direction registers

        clrf    PORTA               ; clear PORTA output latches
        clrf    PORTC               ; clear PORTC output latches

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        movlw   PORTATRIS
        movwf   TRISA               ; Write to TRISA register

        movlw   PORTCTRIS
        movwf   TRISC               ; Write to TRISC register

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; PORTA Weak Pull-up Register (WPUA) (Section 4.2.1)
;
; Each of the PORTA pins, except RA3, has an individually configurable
; weak internal pull-up. Control bits WPUAx enable or disable each 
; pull-up. Refer to Register 4-1. Each weak pull-up is automatically
; turned off when the port pin is configured as an output. The pull-ups
; are disabled on a Power-on Reset by the /RAPU bit (see OPTION register
; below).

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

;    PORTA Pins = xx54x210
        movlw   B'00000010'         ; PORTA 1 weak pull-up ENABLED
        movwf   WPUA

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; PORTA Interrupt-on-Change Register (IOCA) (Section 4.2.2)
;
; Each of the PORTA pins is individually configurable as an interrupt-
; on-change pin. Control bits IOCAx enable or disable the interrupt 
; function for each pin. Refer to Register 4-4. The interrupt-on-change
; is disabled on a Power-on Reset.
;
; Note: Global, port change, and peripheral interrupt enables 
;       (GIE, RAIE and PEIE) must be enabled for individual interrupts
;       to be recognized (see INTCON register below).

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

;    PORTA Pins = xx54x210
        movlw   B'00000010'         ; PORTA 1 interrupt-on-change ENABLED
        movwf   IOCA

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; OPTION Register (OPTION_REG) (Section 2.2.2.2)
;
; The OPTION_REG contains control bits to configure:
;   Weak pull-ups on PORTA (see also WPUA Register above)
;   External RA2/INT interrupt
;   TMR0
;   TMR0/WDT prescaler

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     OPTION_REG, NOT_RAPU    ; PORTA pull-ups: ENABLED

        bsf     OPTION_REG, INTEDG  ; Interrupt Edge: on rising edge of RA2/INT pin

; TIMER0 Module (Section 5.0)
;
; The Timer0 module timer/counter has the following features:
;   8-bit timer/counter
;   Readable and writable
;   8-bit software programmable prescaler
;   Internal or external clock select
;   Interrupt on overflow from 0xFF to 0x00
;   Edge select for external clock

        bcf     OPTION_REG, T0CS    ; TMR0 Clock Source: internal instruction cycle (CLKOUT)
        bcf     OPTION_REG, T0SE    ; TMR0 Source Edge: increment low-to-high transition on GP2/T0CKI pin

        bcf     OPTION_REG, PSA     ; Prescaler Assignment: assigned to TIMER0

        ; TMR0 Prescaler Rate: 1:2
        bcf     OPTION_REG, PS2     ; Prescaler Rate Select bit 2
        bcf     OPTION_REG, PS1     ; Prescaler Rate Select bit 1
        bcf     OPTION_REG, PS0     ; Prescaler Rate Select bit 0

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; TIMER1 Module with Gate Control (Section 6.0)
;
; Timer1 has the following features:
;   16-bit timer/counter (TMR1H:TMR1L)
;   Readable and writable
;   Internal or external clock selection
;   Synchronous or asynchronous operation
;   Interrupt on overflow from 0xFFFF to 0x0000
;   Wake-up upon overflow (Asynchronous mode)
;   Optional external enable input
;    - Selectable gate source; T1G or C2 output (T1GSS (COMCON1<>))
;    - Selectable gate polarity (T1GINV (T1CON<7>))
;   Optional LP oscillator
; The TIMER1 Control Register (T1CON) is used to enable/disable TIMER1
; and select various features of the TIMER1 module.

        bcf     T1CON, TMR1CS       ; TIMER1 Clock Source Select: internal clock (Fosc/4)

        bsf     T1CON, NOT_T1SYNC   ; TIMER1 External Clock Input Sync Control: do not syncronize external clock input

        ; T1OSCEN only if INTOSC without CLKOUT oscillator is active, else ignored
        bcf     T1CON, T1OSCEN      ; LP Oscillator Enable Control: LP oscillator is off

        ; TIMER1 Input Prescale Select: 1:8
        bsf     T1CON, T1CKPS1      ; TIMER1 Input Clock Prescale Select bit 1
        bsf     T1CON, T1CKPS0      ; TIMER1 Input Clock Prescale Select bit 0

        ; TMR1GE only if TMR1ON = 1, else ignored
        bcf     T1CON, TMR1GE       ; TIMER1 Gate Enable: Timer 1 is on (always)

        bcf     T1CON, T1GINV       ; TIMER1 gate is not inverted (not used)

        bcf     T1CON, TMR1ON       ; TIMER1 stopped


;----------------------------------------
; TIMER2 Module with Gate Control (Section 7.0)
;
; The Timer2 module timer has the following features:
;   8-bit timer (TMR2 register)
;   8-bit period register (PR2)
;   Readable and writeable (both registers)
;   Software programmable prescaler (1:1, 1:4, 1:16)
;   Software programmable postscaler (1:1 to 1:16)
;   Interrupt on TMR2 match with PR2

        ; Timer2 Output Postscale Select: 1:1 Postscale
        bcf     T2CON, TOUTPS3      ; Timer2 Input Clock Prescale Select bit 3
        bcf     T2CON, TOUTPS2      ; Timer2 Input Clock Prescale Select bit 2
        bcf     T2CON, TOUTPS1      ; Timer2 Input Clock Prescale Select bit 1
        bcf     T2CON, TOUTPS0      ; Timer2 Input Clock Prescale Select bit 0

        bcf     T2CON, TMR2ON       ; Timer2: Off

        ; Timer2 Clock Prescale Select: 1
        bcf     T2CON, T2CKPS1      ; Timer2 Clock Prescale Select bit 1
        bcf     T2CON, T2CKPS0      ; Timer2 Clock Prescale Select bit 0


;----------------------------------------
; Enhanced Capture/Compare/PWM (ECCP) Module (Section 8.0)
;
; The ECCP module contains a 16-bit register which can operate as a:
;   16-bit Capture register
;   16-bit Compare register
;   PWM Master/Slave Duty Cycle register
; Capture/Compare/PWM Register1 (CCPR1) is comprised of two 8-bit 
; registers: CCPR1L (low byte) and CCPR1H (high byte). The CCP1CON
; register controls the operation of the ECCP. The special event
; trigger is generated by a compare match and will clear both TMR1H 
; and TMR1L registers.

        ; PWM Output Configuration: 
        ; If CCP1M<3:2> = 00, 01, 10
        ;   xx = P1A assigned as Capture/Compare input; P1B, P1C, P1D assigned as port pins
        ; If CCP1M<3:2> = 11
        ;   00 = Single output; P1A modulated; P1B, P1C, P1D assigned as port pins
        ;   01 = Full-bridge output forward; P1D modulated; P1A active; P1B, P1C inactive
        ;   10 = Half-bridge output; P1A, P1B modulated with dead band control;
        ;                            P1C, P1D assigned as port pins
        ;   11 = Full-bridge output reverse; P1B modulated; P1C active; P1A, P1D inactive
        bcf CCP1CON, P1M1           ; PWM Output Configuration bit 1
        bcf CCP1CON, P1M0           ; PWM Output Configuration bit 0

        ; ECCP Mode Select: 0000 = Capture/Compare/PWM off (resets ECCP module)
        bcf CCP1CON, CCP1M3         ; ECCP Mode Select bit 3
        bcf CCP1CON, CCP1M2         ; ECCP Mode Select bit 2
        bcf CCP1CON, CCP1M1         ; ECCP Mode Select bit 1
        bcf CCP1CON, CCP1M0         ; ECCP Mode Select bit 0

        ;PWM Configuration Register

        bcf PWM1CON, PRSEN          ; PWM Restart Enable: Upon auto-shutdown, ECCPASE must be cleared in software to restart the PWM

        bcf PWM1CON, PDC6           ; PWM Delay Count bit 6
        bcf PWM1CON, PDC5           ; PWM Delay Count bit 5
        bcf PWM1CON, PDC4           ; PWM Delay Count bit 4
        bcf PWM1CON, PDC3           ; PWM Delay Count bit 3
        bcf PWM1CON, PDC2           ; PWM Delay Count bit 2
        bcf PWM1CON, PDC1           ; PWM Delay Count bit 1
        bcf PWM1CON, PDC0           ; PWM Delay Count bit 0

        ; Enhanced Capture/Compare/PWM Auto-Shutdown Control Register

        ; ECCP Auto-Shutdown Source Select: Auto-shutdown is disabled
        bcf ECCPAS, ECCPAS2         ; ECCP Auto-Shutdown Source Select bit 2
        bcf ECCPAS, ECCPAS1         ; ECCP Auto-Shutdown Source Select bit 1
        bcf ECCPAS, ECCPAS0         ; ECCP Auto-Shutdown Source Select bit 0

        ; Pin A and C Shutdown State Control: 00 = Drive Pins A and C to '0'
        bcf ECCPAS, PSSAC1          ; Pin A and C Shutdown State Control bit 1
        bcf ECCPAS, PSSAC0          ; Pin A and C Shutdown State Control bit 0

        ; Pin B and D Shutdown State Control: 00 = Drive Pins B and D to '0'
        bcf ECCPAS, PSSBD1          ; Pin B and D Shutdown State Control bit 1
        bcf ECCPAS, PSSBD0          ; Pin B and D Shutdown State Control bit 0


;----------------------------------------
; Comparator Module (Section 9.0)
;
; The PIC16F684 comparator module contains two analog comparators.
; The inputs to the comparator are multiplexed with the RA0 and RA1 pins.
; There is an on-chip Comparator Voltage Reference that can also be applied
; to an input of the comparator. In addition, RA2 can be configured as the
; comparator output. The Comparator Control Register (CMCON) contains bits
; to control the comparator. The Voltage Reference Control Register (VRCON)
; controls the voltage reference module.

        ; Comparator Configuration (Figure 9-2)
;        bcf     CMCON, CINV         ; Comparator Output Inversion: not inverted
;        bcf     CMCON, COUT         ; Comparator Output bit: Vin+ < Vin-
;        bcf     CMCON, CIS          ; Comparator Input Switch: Vin- connects to Cin-

        ; CM2:CM0 = 111 - Comparator Off (lowest power)
        bsf     CMCON0, CM2         ; Comparator Mode bit 2
        bsf     CMCON0, CM1         ; Comparator Mode bit 1
        bsf     CMCON0, CM0         ; Comparator Mode bit 0

        ; VRCON (Register 9-2)
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     VRCON, VREN         ; CVref circuit: powered down, no Idd drain

;        bcf     VRCON, VRR          ; CVref Range Selection: High Range

;        bcf     VRCON, VR3          ; CVref value selection bit 3
;        bcf     VRCON, VR2          ; CVref value selection bit 2
;        bcf     VRCON, VR1          ; CVref value selection bit 1
;        bcf     VRCON, VR0          ; CVref value selection bit 0

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; Analog-to-Digital Converter (A/D) Module (Section 10.0)
;
; The analog-to-digital converter (A/D) allows conversion of an analog
; input signal to a 10-bit binary representation of that signal. The
; PIC16F684 has eight analog inputs multiplexed into one sample and hold
; circuit. There are three registers to control the functions of the A/D
; module:
;   A/D Control Register 0 (ADCON0)
;   A/D Control Register 1 (ADCON1)
;   Analog Select Register (ANSEL)
;
; Note: When using PORTA or PORTC pins as analog inputs, ensure the 
;       TRISA or TRISC register bits are set (= 1) for input.

        bsf     ADCON0, ADFM        ; A/D Result Formed: right justified
        bcf     ADCON0, VCFG        ; Voltage Reference: Vdd

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        ; select A/D Conversion Clock Source: Fosc/16
        bsf     ADCON1, ADCS2       ; A/D Conversion Clock Select bit 2
        bcf     ADCON1, ADCS1       ; A/D Conversion Clock Select bit 1
        bsf     ADCON1, ADCS0       ; A/D Conversion Clock Select bit 0

        ; select PORT pins that will be analog inputs: 
        bcf     ANSEL, ANS7         ; Analog Select RC3/AN7: digital I/O
        bcf     ANSEL, ANS6         ; Analog Select RC2/AN6: digital I/O
        bcf     ANSEL, ANS5         ; Analog Select RC1/AN5: digital I/O
        bsf     ANSEL, ANS4         ; Analog Select RC0/AN4: analog input
        bcf     ANSEL, ANS3         ; Analog Select RA3/AN3: digital I/O
        bcf     ANSEL, ANS2         ; Analog Select RA2/AN2: digital I/O
        bcf     ANSEL, ANS1         ; Analog Select RA1/AN1/Vref: digital I/O
        bcf     ANSEL, ANS0         ; Analog Select RA0/AN0: digital I/O

        bcf     STATUS, RP0         ;---- Select Bank 0 -----

;        bcf     ADCON0, ADON        ; ADC is shut-off and consumes no operating current
        bsf     ADCON0, ADON        ; ADC is on


;----------------------------------------
; Peripheral Interrupt Enable Register (PIE1) (Section 2.2.2.4)
;
; The PIE1 register contains peripheral interrupt enable bits.
;
; NOTE: The PEIE bit must be set to enable any peripheral interrupt 
;       (see INTCON register below).

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     PIE1, EEIE          ; EE Write Complete Interrupt: disabled
        bcf     PIE1, ADIE          ; A/D Converter Interrupt: disabled
        bcf     PIE1, CCP1IE        ; 
        bcf     PIE1, C2IE          ;
        bcf     PIE1, C1IE          ;
        bcf     PIE1, OSFIE         ;
        bcf     PIE1, TMR2IE        ; TMR2 Overflow Interrupt: disabled
        bcf     PIE1, TMR1IE        ; TMR1 Overflow Interrupt: disabled

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; Interrupt Control Register (INTCON) (Section 2.2.2.3)
;
; The INTCON register contains enable and disable flag bits for 
; TMR0 register overflow, PORTA port change and external RA2/INT pin 
; interrupts.

        bcf     INTCON, T0IE        ; TMR0 Overflow Interrupt: disabled
        bcf     INTCON, INTE        ; RA2/INT External Interrupt: disabled
        bsf     INTCON, RAIE        ; Port Change Interrupt: ENABLED

        bcf     INTCON, PEIE        ; Peripheral Interrupts: disabled
                                    ; (EEI, ADI, CCP1I, C2I, C1I, OSFI, TMR2I, TMR1I)

        bcf     INTCON, GIE         ; Global Interrupts: disabled

        return                      ; return from INITIALIZE


;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
