;----------------------------------------------------------------------
; 
;                     Software License Agreement
; 
; The software supplied herewith by Microchip Technology Incorporated
; (the "Company") for its PICmicro Microcontroller is intended and
; supplied to you, the Companys customer, for use solely and
; exclusively on Microchip PICmicro Microcontroller products.
; 
; The software is owned by the Company and/or its supplier, and is
; protected under applicable copyright laws. All rights are reserved.
; Any use in violation of the foregoing restrictions may subject the
; user to criminal sanctions under applicable laws, as well as to
; civil liability for the breach of the terms and conditions of this
; license.
; 
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
; WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
; TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
; PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
; IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
; 
;----------------------------------------------------------------------
;
;   Filename:           main.asm
;   Microcontroller:    PIC16F684
;   Assembled using:    MPLAB IDE v6.50
;
;   Author:             Steven Bible
;   Company:            Microchip Technology Inc.
;
;----------------------------------------------------------------------
;
;   Program Description:
;
;   < type program description here >
;
;----------------------------------------------------------------------
;
;   Revision History:
;
;   Ver. 0.0 - August 2, 2004       initial write
;   Ver. 1.0 - August 10, 2004
;
;----------------------------------------------------------------------
; Include and Project Files:
;
;       16f684.lkr                  ; PIC16F684 Linker file
;       project.inc                 ; I/O port defines and processor defines
;       initialize_f684.asm         ; Initialization routine for PIC16F684
;       ssc.asm                     ; Synchronous Serial Communications subroutines
;       spi.asm                     ; Serial Peripheral Interface subroutines
;       therm design 1.inc          ; Thermistor subroutines include file
;       therm design 1.asm          ; Thermistor subroutines 
;       pga.inc                     ; Programmable Gain Amplifier subroutines include file
;       pga.asm                     ; Programmable Gain Amplifier subroutines
;

#include    "project.inc"           ; io port and processor defines
#include    "pga.inc"

    errorlevel  -302                ; suppress message 302 from list file

; ---------------------------------------------------------------------
; Configuration Bits (Section 12.1)
;
;   The Configuration Bits select various device configurations.
;   These bits are mapped in program memory location 0x2007.
;
;   Note: Address 2007h is beyond the user program memory space. It
;   belongs to the special configuration memory space (0x2000 - 0x3FFF),
;   which can be accessed only during programming. See PIC16F684
;   Memory Programming Specification (DS41204) for more information.
; ---------------------------------------------------------------------
;
; Fail Clock Monitor Enabled bit:
; _FCMEN_ON = Enabled
; _FCMEN_OFF = Disabled
;
; Internal External Switch Over bit:
; _IESO_ON = Enabled
; _IESO_OFF = Disabled
;
; Brown-out Detect Selection bits:
; Note: Enabling Brown-out Detect (BOD) does not automatically enable 
;       Power-up Timer
; _BOD_ON = Enabled
; _BOD_NSLEEP = Enabled during operation and disabled in SLEEP
; _BOD_SBODEN = Controlled by SBODEN bit (PCON<4>)
; _BOD_OFF = Disabled
;
; Data Memory Code Protection bit:
; Note: The entire data EEPROM will be erased when the code protection
;       is turned off.
; _CPD_ON = Enabled
; _CPD_OFF = Disabled
;
; Program Memory Code protection:
; Note: The entire program memory will be erased when the code protection
;       is turned off.
; _CP_ON = Enabled
; _CP_OFF = : Disabled
;
; RA3/MCLR pin function select:
; NOTE: When /MCLR is asserted in INTOSC or RC mode, the internal clock 
;       oscillator is disabled.
; _MCLRE_ON = RA3/MCLR pin function is /MCLR
; _MCLRE_OFF = RA3/MCLR pin function is digital I/O, 
;              /MCLR internally tied to Vdd
;
; Power-up Timer Enable bit:
; _PWRTE_ON = Enabled
; _PWRTE_OFF = Disabled
;
; Watchdog Timer Enable bit:
; _WDT_ON = Enabled
; _WDT_OFF = Disabled
;
; Oscillator Selction bits:
; NOTE: When /MCLR is asserted in INTOSC or RC mode, the internal clock
;       oscillator is disabled.
; _EXTRC_OSC_CLKOUT = CLKOUT function on RA4 pin, RC on RA5 pin.
; _EXTRC_OSC_NOCLKOUT = I/O function on RA4 pin, RC on RA5 pin.
; _INTRC_OSC_CLKOUT =  Internal oscillator, CLKOUT function on RA4 pin,
;                      I/O function on RA5 pin.
; _INTRC_OSC_NOCLKOUT = Internal oscillator, I/O function on RA4 and RA5 pins.
; _EC_OSC = I/O function on RA4 pin, CLKIN on RA5 pin.
; _HS_OSC = High speed crystal/resonator on RA4 and RA5 pins.
; _XT_OSC = Crystal/resonator on RA4 and RA5 pins.
; _LP_OSC = Low power crystal on RA4 and RA5 pins.
;
; ---------------------------------------------------------------------

    __CONFIG    _FCMEN_OFF & _IESO_OFF & _BOD_OFF & _CPD_OFF & _CP_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT

;----------------------------------------------------------------------
; Externally Defined Labels
;
; EXTERN declares symbol names that are used in this module and declared
; as global in another module.
;----------------------------------------------------------------------

    ;--------------------
    ; external variables

    extern  Therm_PwLI_IN           ; from therm.asm
    extern  Therm_PwLI_OUT          ; from therm.asm

    extern  PGAgain                 ; from pga.asm
    extern  PGAadc                  ; from pga.asm
;    extern  PGAtriplow              ; from pga.asm
;    extern  PGAtriphigh             ; from pga.asm

    extern  SSC_RxByte              ; from ssc.asm

    ;--------------------
    ; external subroutines

    extern  initialize_f684         ; from initialize_f684.asm

    extern  Therm_PwLI              ; from therm.asm

    extern  PGA_SetGain             ; from pga.asm
    extern  PGA_SetChan             ; from pga.asm
    extern  PGA_Shutdown            ; from pga.asm

    extern  SSC_GetByte             ; from ssc.asm
    extern  SSC_SendByte            ; from ssc.asm

;----------------------------------------------------------------------
; Global Defines
;----------------------------------------------------------------------

; see project.inc

;----------------------------------------------------------------------
; Local Defines
;----------------------------------------------------------------------

    ; MCP6S22 channel assignments

#define Therm_Chan_0 0
;#define Sensor_Chan_1 1

    ; Hysteresis high and low limits

;#define LOW_VAL     D'226'
;#define HIGH_VAL    D'960'

    ; Flag bits

;#define flag_bit_0 0
;#define flag_bit_1 0
;#define flag_bit_2 0
;#define flag_bit_3 0
;#define flag_bit_4 0
;#define flag_bit_5 0
;#define flag_bit_6 0
;#define flag_bit_7 0


;----------------------------------------------------------------------
; Variables  (Section 2.2 Data Memory Organization)
;----------------------------------------------------------------------

    ; Data Memory Organization (Section 2.2)
    ;
    ; The data memory is partitioned into two banks which contain
    ; the General Purpose Registers and the Special Function Registers.
    ; The Special Function registers are located in the first 32 
    ; locations of each bank. Register locations 0x20 to 0x5F (64 bytes)
    ; in Bank 0 and 0xA0 to 0xBF in Bank 1 are General Purpose registers,
    ; implemented as static RAM. Register locations 0xFH to 0xFF in 
    ; Bank 1 point to addresses 0x70 to 0x7F in Bank 0. All other RAM is
    ; unimplemented and returns '0' when read. RP0 (STATUS<5>) is the 
    ; bank select bit.
    ;
    ;   RP0 (STATUS<5>)
    ;    0  ->  Bank 0
    ;    1  ->  Bank 1
    ;
    ; Refer to Section 2.2 of the data sheet for the organization of 
    ; the General Purpose Registers.

;--------------------
; Uninitialized Data Section
;
; The RES directive reserves space for data storage.

        udata

TempISRdata     res 1               ; temporary variable for ISR routine
TempADCdata     res 1               ; temporary variable for ADC subroutine
TempMainLoop    res 2               ; temporary variable for Main subroutine

DelayCounter    res 1               ; delay counter variable

CmdStatus       res 1               ; command status

Therm           res 2               ; Thermistor variable


;--------------------
; Shared Uninitialized Data Section
;
; Variables that are allocated in RAM that are shared across all RAM banks (unbanked RAM). 
; The RES directive reserves space for data storage.
;
        udata_shr

W_TEMP          res 1               ; used for context saving 
STATUS_TEMP     res 1               ; used for context saving
PCLATH_TEMP     res 1               ; used for context saving
FSR_TEMP        res 1               ; used for context saving

;Flags           res 1               ; a byte of flag bits (see local define for bit definitions above)

DelayTimer      res 1               ; DelayTimer variable

;--------------------
; Global Variables
;
; Variables defined in this module that should be available to other
; modules. The <label> must be defined prior to using the GLOBAL directive. 

    ; none


;----------------------------------------------------------------------
; Begin Executable Code Segment
;----------------------------------------------------------------------

;[<label>]   code    [<ROM_Address>]

        ; Program Memory Organization (Section 2.1)

ResetVector     code    0x0000
  
        nop                         ; for ICD use
        goto    MAIN                ; goto MAIN

        ; interrupt vector location

InterruptVector code    0x0004

        movwf   W_TEMP              ; save W register
        swapf   STATUS, W           ; swap status to be saved into W
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movwf   STATUS_TEMP         ; save STATUS register
        movfw   PCLATH
        movwf   PCLATH_TEMP         ; save PCLATH_TEMP register
        movfw   FSR
        movwf   FSR_TEMP            ; save FSR_TEMP register

;----------------------------------------
; Interrupt Service Routine (ISR) (Section 9.4)
; Description: Arrange the order of the interrupt flag bit tests for
;   highest priority at top to lowest priority at bottom.

RAIF_ISR        ; Interrupt-on-change (PORTA 1) Interrupt Service Routine

        btfss   INTCON, RAIF        ; check PORT Change interrupt flag
         goto   End_ISR

        ; The PORT Change interrupt can be cleared by a reading PORTA.
        ; This will end the mismatch condition that caused the interrupt
        ; and clear the RAIF bit. It appears that the best place to do
        ; this is at the end of the ISR routine otherwise the interrupt
        ; may not clear.

        call    SSC_GetByte

        andlw   0x07                ; mask out lower 3 bits

        movwf   TempISRdata
        movlw   HIGH cmdTable
        movwf   PCLATH
        movfw   TempISRdata
        addlw   LOW cmdTable
        btfsc   STATUS, C
         incf   PCLATH, F
        movwf   PCL

cmdTable
        goto    cmdVersion          ; 0x00 - VERSION      ( cmd: 1 / arg: 0 )
        goto    GetRTSample         ; 0x01 - get an Real Time sample
        goto    End_ISR
        goto    End_ISR
        goto    End_ISR
        goto    End_ISR
        goto    End_ISR
        goto    cmdGetStatus        ; 0x07 - GETSTATUS    ( cmd: 1 / arg: 1 )

;----------------------------------------
; Returns a version number for the firmware
; ( cmd: 1 / arg: 0 )

cmdVersion

        movlw   .75                 ; take some time to turn the bus around
        movwf   DelayCounter
        decfsz  DelayCounter, F
         goto   $ - 1

        movlw   VERSION
        call    SSC_SendByte

        movlw   CMD_Version_SUCCESSFUL
        movwf   CmdStatus

        movfw   PORTA               ; clear any interrupt on change conditions
        bcf     INTCON, RAIF

        goto    End_ISR

;----------------------------------------
; Get Real Time Sample

GetRTSample

        call    READ_ANALOG_PGA     ; read ADC

        movlw   .75                 ; take some time to turn the bus around
        movwf   DelayCounter
        decfsz  DelayCounter, F
         goto   $ - 1

        bsf     STATUS, RP0         ; --- Select Bank 1 ---
        movfw   ADRESL
        bcf     STATUS, RP0         ; --- Select Bank 0 ---
        movwf   Therm_PwLI_IN

        movfw   ADRESH
        movwf   Therm_PwLI_IN+1

        call    Therm_PwLI          ; perform piecewise linear interpolation

        movfw   Therm_PwLI_OUT+1    ; output is degrees C
        movwf   Therm+1
        call    SSC_SendByte        ; send high byte

        movfw   Therm_PwLI_OUT
        movwf   Therm
        call    SSC_SendByte        ; send low byte

        goto    End_ISR

;----------------------------------------
; Return command status
; ( cmd: 1 / arg: 0 )

cmdGetStatus

        movlw   .75                 ; take some time to turn the bus around
        movwf   DelayCounter
        decfsz  DelayCounter, F
         goto   $ - 1

        movfw   CmdStatus           ; report that last status message
        call    SSC_SendByte

        movlw   CMD_UNSUCCESSFUL    ; clear status message
        movwf   CmdStatus

        movfw   PORTA               ; clear any interrupt on change conditions
        bcf     INTCON, RAIF

;        goto    End_ISR

;----------------------------------------
End_ISR

        movfw   PCLATH_TEMP         ; restore PCLATH_TEMP register
        movwf   PCLATH
        movfw   FSR_TEMP            ; restore FSR_TEMP register
        movwf   FSR
        swapf   STATUS_TEMP, W      ; swap status_temp into W, sets bank to original state
        movwf   STATUS              ; restore STATUS register
        swapf   W_TEMP, F
        swapf   W_TEMP, W           ; restore W register

        retfie

;----------------------------------------
; Program Subroutines
;----------------------------------------

;----------------------------------------------------------------------
; Subroutine: DATA_EEPROM_READ
;
; Description: To read an EEPROM data memory location, the address is
;   written to the EEADR register and set control bit RD (EECON1<0>) to
;   initiate a read. Data is available in the EEDATA register the next
;   clock cycle.
;
; Constants: none
;   
; Global Variables: none
;   
; Initialization: W contains EEPROM address (EEADR) to be read
;   
; Output: W contains EEPROM data (EEDATA)
;
;----------------------------------------------------------------------

DATA_EEPROM_READ

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        movwf   EEADR               ; move EEPROM address in W to EEADR
        bsf     EECON1, RD          ; initiate EEPROM read
        movf    EEDAT, W            ; move data to W

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        return

;----------------------------------------------------------------------
; Subroutine: DATA_EEPROM_WRITE
;
; Description: To write an EEPROM data memory location, the address is
;   written to the EEADR register, data to the EEDATA register, then
;   execute a required sequence of instructions.
;   
; CAUTION: Global Interrupts (GIE) are disable and then re-enabled 
;          during this subroutine.
;
; Constants: none
;   
; Global Variables: none
;   
; Initialization: Address = EEADR, Data = EEDATA
;   
; Output: none
;
;----------------------------------------------------------------------

DATA_EEPROM_WRITE

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bsf     EECON1, WREN        ; EEPROM Write Enable: allow write cycles
        bcf     INTCON, GIE         ; disable global interrupts
                                    ; *** required sequence, do not alter ***
        movlw   0x55
        movwf   EECON2
        movlw   0xAA
        movwf   EECON2
        bsf     EECON1, WR          ; initiate EEPROM write
                                    ; *** end required sequence ***

        btfsc   EECON1, WR          ; has write completed?
        goto    $-1

        bsf     INTCON, GIE         ; enable global interrupts
        bcf     EECON1, WREN        ; EEPROM Write Enable: inhibit write cycles

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        return

;----------------------------------------------------------------------
; Subroutine: READ_ANALOG_PGA
;
; Description: Read analog channel
;   
; Constants: none
;   
; Global Variables: none
;   
; Initialization: none
;   
; Output: ADRESH and ADRESL contain 10-bit A/D result justified 
;   according to ADCON0, ADFM bit.
;
;----------------------------------------------------------------------

READ_ANALOG_PGA

        bsf     ADCON0, CHS2        ; select analog channel AN4
        bcf     ADCON0, CHS1
        bcf     ADCON0, CHS0

;        bsf     ADCON0, ADON        ; Turn on ADC module

        ; After selecting a new channel, allow for sufficent sample time.
        ; The amount of sample time depends on the charging time of the
        ; internal charge holding capacitor (Section 7.2).

        movlw   D'14'                ; At 8 MHz, a ~22 us delay
        movwf   TempADCdata         ; (22.5us = 1us + 14 * 1.5us + .5us)
        decfsz  TempADCdata, F
        goto    $-1

        bsf     ADCON0, GO          ; start A/D conversion

;        sleep                       ; Must select Frc to use Sleep

        btfsc   ADCON0, GO          ; has A/D conversion completed?
        goto    $-1

;        bcf     ADCON0, ADON        ; Turn off ADC module (consumes no operating current)

        return

;----------------------------------------------------------------------
; Subroutine: 
;   
; Description: 
;   
; Constants: 
;   
; Global Variables: 
;   DelayTimer
;   
; Initialization: 
;   
; Output: 
;   
;----------------------------------------------------------------------



;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Main Program
;----------------------------------------------------------------------
;----------------------------------------------------------------------

MAIN

;----------------------------------------
; Initialize PICmicro
;----------------------------------------

        call    initialize_f684

;----------------------------------------
; Initialize Variables
;----------------------------------------

        bcf     SCK                 ; SPI Serial Clock LOW

        bsf     PGA_CS              ; de-select PGA

        movlw   PGA_GAIN_1          ; PGA gain to +1 (gain select time is 1us typ)
        movwf   PGAgain
        call    PGA_SetGain

        movlw   Therm_Chan_0        ; PGA channel to 0 (channel select time is 1.5us typ)
        call    PGA_SetChan

;        call    PGA_Shutdown        ; shutdown the PGA

;        clrf    Flags               ; all flag bits = FALSE

        movlw   CMD_UNSUCCESSFUL
        movwf   CmdStatus

;        movlw   HIGH LOW_VAL        ; initialize low PGA hysteresis trip value
;        movwf   PGAtriplow+1
;        movlw   LOW LOW_VAL
;        movwf   PGAtriplow

;        movlw   HIGH HIGH_VAL       ; initialize high PGA hysteresis trip value
;        movwf   PGAtriphigh+1
;        movlw   LOW HIGH_VAL
;        movwf   PGAtriphigh

        movfw   PORTA               ; clear any interrupt on change conditions
        bcf     INTCON, RAIF

        bsf     INTCON, GIE         ; enable global interrupts

MAINLOOP

        sleep
        nop

        goto    MAINLOOP


;----------------------------------------------------------------------
; Data EEPROM Memory (Section 8.0)
;
; PIC16F684 has 256 bytes of data EEPROM with address
; range 0x00 to 0xFF (0x2100 to 0x21FF).

        ; Initialize Data EEPROM Memory locations

;DEEPROM code
;        DE      0x00, 0x01, 0x02, 0x03

;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
