;----------------------------------------------------------------------
;
;   Filename:           pga.asm
;   Microcontroller:    PIC16F684
;   Assembled using:    MPLAB IDE v6.40
;
;   Author:             Steven Bible
;   Company:            Microchip Technology Inc.
;
;----------------------------------------------------------------------
;
;   Program Description:
;
;   < type program description here >
;
;----------------------------------------------------------------------
;
;   Revision History:
;
;   Ver. 0.0 - March 18, 2004: Initial write - sb
;   Ver. 0.1 - August 2004: Added PGA_Hysteresis - sb
;   Ver. 0.1a - September 2004: Modified PGA_Hysteresis for AN897 - sb
;
;----------------------------------------------------------------------
; Include Files and Project Files:
;

#include    "project.inc"           ; io port and processor defines
#include    "pga.inc"

    errorlevel  -302                ; suppress message 302 from list file

;----------------------------------------------------------------------
; Externally Defined Labels
;
; EXTERN declares symbol names that are used in this module and declared
; as global in another module.
;----------------------------------------------------------------------

    ; external variables

    ; none

    ; external subroutines

    extern  SPI_PutByte             ; spi.asm
    extern  SPI_GetByte             ; spi.asm

;----------------------------------------------------------------------
; Global Defines
;----------------------------------------------------------------------

; see project.inc

;----------------------------------------------------------------------
; Local Defines
;----------------------------------------------------------------------

    ; none

;----------------------------------------------------------------------
; Variables
;----------------------------------------------------------------------

;--------------------
; Uninitialized Data Section
;
; The RES directive reserves space for data storage.

        udata

PGAdata     res 1                   ; PGA data variable (local)

PGAadc      res 2                   ; ADC reading variable (global)

;PGAtriplow  res 2                   ; low hysteresis trip value (global)
;PGAtriphigh res 2                   ; high hysteresis trip value (global)


;--------------------
; Shared Uninitialized Data Section
;
; Variables that are allocated in RAM that is shared across all RAM banks (unbanked RAM). 
; The RES directive reserves space for data storage.
;
; PIC16F684 has 16 shared RAM locations
;
        udata_shr

PGAgain     res 1                   ; PGA gain variable (global)


;--------------------
; Global Variables
;
; Variables defined in this module that are made available to other 
; modules. The <label> must be defined prior to using the GLOBAL
; directive in this module. 

    global  PGAgain
    global  PGAadc
;    global  PGAtriplow
;    global  PGAtriphigh


;----------------------------------------------------------------------
; Begin Executable Code Segment
;----------------------------------------------------------------------

;[<label>]   code    [<ROM_Address>]

        code

;----------------------------------------------------------------------
; Subroutine: Set PGA Gain
;   
; Description: Sets the gain setting in the PGA
;   
; Constants: 
;   PGA gain settings (defined in pga.inc):
;       PGAgain_1 = Gain +1
;       PGAgain_2 = Gain +2
;       PGAgain_4 = Gain +4
;       PGAgain_5 = Gain +5
;       PGAgain_8 = Gain +8
;       PGAgain_10 = Gain +10
;       PGAgain_16 = Gain +16
;       PGAgain_32 = Gain +32
;
; Global Variables: 
;   
; Initialization: 
;   1. W register contains the PGA gain setting to send
;   
; Output: 
;   None
;   
;----------------------------------------------------------------------

PGA_SetGain
        global  PGA_SetGain

        movwf   PGAdata             ; save PGA gain value

        bcf     PGA_CS              ; select PGA

        movlw   0x40                ; send PGA Gain Instruction
        call    SPI_PutByte

        movfw   PGAdata             ; send PGA gain value
        call    SPI_PutByte

        bsf     PGA_CS              ; deselect PGA

        return


;----------------------------------------------------------------------
; Subroutine: Set PGA Channel
;   
; Description: Sets the channel in the PGA
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: 
;   1. W register contains the PGA channel setting to send
;   
; Output: 
;   None
;   
;----------------------------------------------------------------------

PGA_SetChan
        global  PGA_SetChan

        movwf   PGAdata             ; save PGA channel value

        bcf     PGA_CS              ; select PGA

        movlw   0x41                ; send PGA Channel Instruction
        call    SPI_PutByte

        movfw   PGAdata             ; send PGA channel value
        call    SPI_PutByte

        bsf     PGA_CS              ; deselect PGA

        return


;----------------------------------------------------------------------
; Subroutine: Shutdown PGA
;   
; Description: Shutdown the PGA
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: 
;   
; Output: 
;   None
;   
;----------------------------------------------------------------------

PGA_Shutdown
        global  PGA_Shutdown

        bcf     PGA_CS              ; select PGA

        movlw   0x20                ; send PGA Shutdown Instruction
        call    SPI_PutByte

        movfw   0x00                ; send null value
        call    SPI_PutByte

        bsf     PGA_CS              ; deselect PGA

        return


;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
