;----------------------------------------------------------------------
;
;   Filename:           spi.asm
;   Microcontroller:    
;   Assembled using:    MPLAB IDE v6.40
;
;   Author:             Steven Bible
;   Company:            Microchip Technology Inc.
;
;----------------------------------------------------------------------
;
;   Program Description:
;
;   < type program description here >
;
;----------------------------------------------------------------------
;
;   Revision History:
;
;   Ver. 0.0 - February 7, 2004: Initial write
;
;----------------------------------------------------------------------
; Include Files and Project Files:
;

#include    "project.inc"           ; io port and processor defines

;----------------------------------------------------------------------
; Externally Defined Labels
;
; EXTERN declares symbol names that are used in this module and declared
; as global in another module.
;----------------------------------------------------------------------

    ; external variables

    ; none

    ; external subroutines

    ; none
    
;----------------------------------------------------------------------
; Global Defines
;----------------------------------------------------------------------

    ; see project.inc

;----------------------------------------------------------------------
; Local Defines
;----------------------------------------------------------------------

    ; none

;----------------------------------------------------------------------
; Variables
;----------------------------------------------------------------------

;--------------------
; Uninitialized Data Section
;
; The RES directive reserves space for data storage.

        udata

SPI_BitCntr res 1                   ; SPI Bit Counter
SPI_Data    res 1                   ; SPI Data variable

;--------------------
; Shared Uninitialized Data Section
;
; Variables that are allocated in RAM that are shared across all RAM banks (unbanked RAM). 
; The RES directive reserves space for data storage.

;        udata_shr

    ; none

;--------------------
; Global Variables
;
; Variables defined in this module that should be available to other
; modules. The <label> must be defined prior to using the GLOBAL directive. 

;    global  <label>


;----------------------------------------------------------------------
; Begin Executable Code Segment
;----------------------------------------------------------------------

;[<label>]   code    [<ROM_Address>]

            code

;----------------------------------------------------------------------
; Subroutine: SPI Put Byte
;   
; Description: 
;   Send (put) the 8-bit byte contained in the W register out the 
;   SPI Serial Out (SO) pin (w.r.t. the PICmicro).
;   
; Constants: 
;   None
;   
; Global Variables: 
;   1. SPI_BitCntr = SPI bit counter variable
;   2. SPI_Data = SPI Data variable
;   
; Initialization: 
;   1. W register contains the 8-bit byte to send
;   2. The SPI device must be selected prior to calling this routine
;   
; Output: 
;   SCK will be low at the end of the routine
;   
;----------------------------------------------------------------------

SPI_PutByte
        global  SPI_PutByte

        movwf   SPI_Data            ; save data to send

        movlw   D'8'                ; set bit counter to 8
        movwf   SPI_BitCntr

SPI_PutByte_Loop

        bcf     SCK                 ; set SCK low

        rlf     SPI_Data, F         ; rotate left into Carry bit

        btfsc   STATUS, C           ; if Carry bit is set
         bsf    MOSI                ;  SO --> high
        btfss   STATUS, C           ; if Carry bit is clear
         bcf    MOSI                ;  SO --> low

        bsf     SCK                 ; SCK rising edge (data is latched)

        decfsz  SPI_BitCntr, F      ; are all bits sent?
         goto   SPI_PutByte_Loop

        bcf     SCK                 ; set SCK low

        return


;----------------------------------------------------------------------
; Subroutine: SPI Get Byte
;   
; Description: 
;   Receive (get) an 8-bit byte from the SPI Serial In (SI) pin (w.r.t.
;   the PICmicro) and place value into the W register.
;   
; Constants: 
;   None
;   
; Global Variables: 
;   1. SPI_BitCntr = SPI bit counter variable
;   2. SPI_Data = SPI Data variable
;
; Initialization: 
;   1. The SPI device must be selected prior to calling this routine
;   2. SCK should be low prior to entering routine
;   
; Output: 
;   1. W register contains received 8-bit byte
;   2. SCK will be low at the end of the routine
;   
;----------------------------------------------------------------------

SPI_GetByte
        global  SPI_GetByte

        movlw   D'8'                ; set bit counter to 8
        movwf   SPI_BitCntr

SPI_GetByte_Loop

        bsf     SCK                 ; SCK rising edge

        btfsc   MISO                ; read SI, if SI = 1
         bsf    STATUS, C           ;  set carry bit
        btfss   MISO                ; read SI, if SI = 0
         bcf    STATUS, C           ;  clear carry bit

        bcf     SCK                 ; set SCK low (data is shifted)

        rlf     SPI_Data, F         ; rotate Carry bit left into SPI_Data

        decfsz  SPI_BitCntr, F      ; all bits received?
         goto   SPI_GetByte_Loop

        movfw   SPI_Data            ; place SPI data received into W

        return


;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
