;----------------------------------------------------------------------
;
;   Filename:           therm.asm
;   Microcontroller:    PIC16F684
;   Assembled using:    MPLAB IDE v6.40
;
;   Author:             John Day
;   Company:            Microchip Technology Inc.
;
;----------------------------------------------------------------------
;
;   Program Description:
;
;   < type program description here >
;
;----------------------------------------------------------------------
;
;   Revision History:
;
;   Ver. 0.0 - March 18, 2004: Initial write
;
;----------------------------------------------------------------------
; Include Files and Project Files:
;

#include    "project.inc"           ; io port and processor defines
#include    "therm design 1.inc"
#include    "pga.inc"

;----------------------------------------------------------------------
; Externally Defined Labels
;
; EXTERN declares symbol names that are used in this module and declared
; as global in another module.
;----------------------------------------------------------------------

    ; external variables



    ; external subroutines



;----------------------------------------------------------------------
; Global Defines
;----------------------------------------------------------------------

    ; see project.inc

;----------------------------------------------------------------------
; Local Defines
;----------------------------------------------------------------------

    ; none

;----------------------------------------------------------------------
; Variables
;----------------------------------------------------------------------

;--------------------
; Uninitialized Data Section
;
; The RES directive reserves space for data storage.

        udata

Therm_PwLI_IN   res 2               ; Input to PwLI subroutine
Therm_PwLI_OUT  res 2               ; 16-bit output from PwLI subroutine

PwLI_IDX        res 2               ; Index into PwLI table
PwLI_SL         res 3               ; 24-bit slope variable
PwLI_MUL        res 3               ; 24-bit multiply result variable

TEMP            res 1               ; temporary variable

;--------------------
; Shared Uninitialized Data Section
;
; Variables that are allocated in RAM that ais shared across all RAM banks (unbanked RAM). 
; The RES directive reserves space for data storage.
;
; PIC16F684 has 16 shared RAM locations
;
;        udata_shr



;--------------------
; Global Variables
;
; Variables defined in this module that are made available to other 
; modules. The <label> must be defined prior to using the GLOBAL
; directive in this module. 

    global  Therm_PwLI_IN
    global  Therm_PwLI_OUT

;----------------------------------------------------------------------
; Begin Executable Code Segment
;----------------------------------------------------------------------

;[<label>]   code    [<ROM_Address>]

        code

;----------------------------------------------------------------------
; Macros:
;----------------------------------------------------------------------

m_add24 macro   input, output       ; output[3] = input[3] + output[3]

        movf    input, W            ; Add input LSB to output LSB
        addwf   output, F
        movf    input+1, W
        btfsc   STATUS, C
        incfsz  input+1, W          ; Add one if there was a carry from previous add
        addwf   output+1, F         ; Add input+1 to output+1
        movf    input+2, W
        btfsc   STATUS, C
        incfsz  input+2, W          ; Add one if there was a carry from previous add
        addwf   output+2, F         ; Add input+2 to output+2

    endm

;----------------------------------------------------------------------
; Subroutine: Piecewise-Linear Interpolation
;   
; Description: 10-bit input, 16-bit output, 64 PL segments, 6 PL bits
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: 
;   
; Output: 
;   
;----------------------------------------------------------------------

Therm_PwLI
        global  Therm_PwLI

        ; calculate PwLI table index (PwLI_IDX) ( 10-bit INPUT; 6-bit PL_TABLE entry )
        ; right shift 3 places (PWLINDEX is multiplied by 2)

        movfw   Therm_PwLI_IN+1
        movwf   PwLI_IDX+1

        movfw   Therm_PwLI_IN
        movwf   PwLI_IDX

        bcf     STATUS, C
        rrf     PwLI_IDX+1, F
        rrf     PwLI_IDX, F

        bcf     STATUS, C
        rrf     PwLI_IDX+1, F
        rrf     PwLI_IDX, F

        bcf     STATUS, C
        rrf     PwLI_IDX+1, F
        rrf     PwLI_IDX, F

        ; at this point, PWLINDEX should contain only 6 bits in positions 6:1

        movlw   0x7E
        andwf   PwLI_IDX,F

; get Y1

        movfw   PwLI_IDX
        call    PwLI_TABLE
        movwf   Therm_PwLI_OUT

        movfw   PwLI_IDX
        addlw   1
        call    PwLI_TABLE
        movwf   Therm_PwLI_OUT+1

; get Y2

        movfw   PwLI_IDX
        addlw   2				
        call    PwLI_TABLE
        movwf   PwLI_SL

        movfw   PwLI_IDX
        addlw   3				
        call    PwLI_TABLE
        movwf   PwLI_SL+1

; calculate intermediate slope value of segment
; subtract Y1 from Y2 (rise within segment), store result in SLOPE

        movfw   Therm_PwLI_OUT
        subwf   PwLI_SL, F
        movf    Therm_PwLI_OUT+1, W
        btfss   STATUS, C
         incfsz Therm_PwLI_OUT+1, W
        subwf   PwLI_SL+1, F
        clrf    PwLI_SL+2
        btfss   STATUS, C
         comf   PwLI_SL+2, F

; calculate Y within segment

        ; multiply lower 4 bits of PwLI_IN times intermediate SLOPE (the full RISE (or FALL) of the segment)

        clrf    PwLI_MUL            ; Clear multiplication result register
        clrf    PwLI_MUL+1
        clrf    PwLI_MUL+2

        btfss   Therm_PwLI_IN, 0      ; Check bit 0 of multiplicand, skip add if it is zero
         goto   skip_add_4
        m_add24 PwLI_SL, PwLI_MUL     ; Add multiplier to result if bit 0 of multiplicand is 1

skip_add_4
        bcf     STATUS, C           ; Clear carry bit in anticipation of a 24-bit rotate
        rlf     PwLI_SL, F            ; Rotate SLOPE[3] one position to the left (multiply by 2)
        rlf     PwLI_SL+1, F
        rlf     PwLI_SL+2, F
        btfss   Therm_PwLI_IN, 1      ; Check bit 1 of multiplicand, skip add if it is zero
         goto   skip_add_3
        m_add24 PwLI_SL, PwLI_MUL    ; Add multiplier to result if bit 1 of multiplicand is 1

skip_add_3	
        rlf     PwLI_SL, F            ; Rotate SLOPE[3] one position to the left (multiply by 2)
        rlf     PwLI_SL+1, F
        rlf     PwLI_SL+2, F
        btfss   Therm_PwLI_IN, 2      ; Check bit 2 of multiplicand, skip add if it is zero
         goto   skip_add_2
        m_add24 PwLI_SL, PwLI_MUL    ; Add multiplier to result if bit 2 of multiplicand is 1

skip_add_2	
        rlf     PwLI_SL, F            ; Rotate SLOPE[3] one position to the left (multiply by 2)
        rlf     PwLI_SL+1, F
        rlf     PwLI_SL+2, F
        btfss   Therm_PwLI_IN, 3      ; Check bit 3 of multiplicand, skip add if it is zero
         goto   skip_add_1
        m_add24 PwLI_SL, PwLI_MUL    ; Add multiplier to result if bit 3 of multiplicand is 1

        ; divide by 16 (now we have Y)

skip_add_1
        swapf   PwLI_MUL, W
        andlw   0x0F
        movwf   PwLI_MUL
        swapf   PwLI_MUL+1, W
        andlw   0xF0
        iorwf   PwLI_MUL, F
        swapf   PwLI_MUL+1, W
        andlw   0x0F
        movwf   PwLI_MUL+1
        swapf   PwLI_MUL+2, W
        andlw   0xF0
        iorwf   PwLI_MUL+1, F

; add Y of the segment to Y1, result in PwLI_OUT
		
        movfw   PwLI_MUL
        addwf   Therm_PwLI_OUT, F
        movfw   PwLI_MUL+1
        btfsc   STATUS, C
         addlw  1
        addwf   Therm_PwLI_OUT+1, F

        return


;----------------------------------------
; Piecewise Linear Interpolation Look-up table

PwLI_TABLE

        movwf   TEMP                ; save W (index into table)
        movlw   HIGH PwLITABLE
        movwf   PCLATH
        movfw   TEMP
        addlw   LOW PwLITABLE
        btfsc   STATUS, C
         incf   PCLATH, F
        movwf   PCL

PwLITABLE
        retlw   low  SEGMENT0_1
        retlw	high SEGMENT0_1
        retlw   low  SEGMENT1_1
        retlw   high SEGMENT1_1
        retlw   low  SEGMENT2_1
        retlw   high SEGMENT2_1
        retlw   low  SEGMENT3_1
        retlw   high SEGMENT3_1
        retlw   low  SEGMENT4_1
        retlw   high SEGMENT4_1
        retlw   low  SEGMENT5_1
        retlw   high SEGMENT5_1
        retlw   low  SEGMENT6_1
        retlw   high SEGMENT6_1
        retlw   low  SEGMENT7_1
        retlw   high SEGMENT7_1
        retlw   low  SEGMENT8_1
        retlw   high SEGMENT8_1
        retlw   low  SEGMENT9_1
        retlw   high SEGMENT9_1
        retlw   low  SEGMENT10_1
        retlw   high SEGMENT10_1
        retlw   low  SEGMENT11_1
        retlw   high SEGMENT11_1
        retlw   low  SEGMENT12_1
        retlw   high SEGMENT12_1
        retlw   low  SEGMENT13_1
        retlw   high SEGMENT13_1
        retlw   low  SEGMENT14_1
        retlw   high SEGMENT14_1
        retlw   low  SEGMENT15_1
        retlw   high SEGMENT15_1
        retlw   low  SEGMENT16_1
        retlw   high SEGMENT16_1
        retlw   low  SEGMENT17_1
        retlw   high SEGMENT17_1
        retlw   low  SEGMENT18_1
        retlw   high SEGMENT18_1
        retlw   low  SEGMENT19_1
        retlw   high SEGMENT19_1
        retlw   low  SEGMENT20_1
        retlw   high SEGMENT20_1
        retlw   low  SEGMENT21_1
        retlw   high SEGMENT21_1
        retlw   low  SEGMENT22_1
        retlw   high SEGMENT22_1
        retlw   low  SEGMENT23_1
        retlw   high SEGMENT23_1
        retlw   low  SEGMENT24_1
        retlw   high SEGMENT24_1
        retlw   low  SEGMENT25_1
        retlw   high SEGMENT25_1
        retlw   low  SEGMENT26_1
        retlw   high SEGMENT26_1
        retlw   low  SEGMENT27_1
        retlw   high SEGMENT27_1
        retlw   low  SEGMENT28_1
        retlw   high SEGMENT28_1
        retlw   low  SEGMENT29_1
        retlw   high SEGMENT29_1
        retlw   low  SEGMENT30_1
        retlw   high SEGMENT30_1
        retlw   low  SEGMENT31_1
        retlw   high SEGMENT31_1
        retlw   low  SEGMENT32_1
        retlw   high SEGMENT32_1
        retlw   low  SEGMENT33_1
        retlw   high SEGMENT33_1
        retlw   low  SEGMENT34_1
        retlw   high SEGMENT34_1
        retlw   low  SEGMENT35_1
        retlw   high SEGMENT35_1
        retlw   low  SEGMENT36_1
        retlw   high SEGMENT36_1
        retlw   low  SEGMENT37_1
        retlw   high SEGMENT37_1
        retlw   low  SEGMENT38_1
        retlw   high SEGMENT38_1
        retlw   low  SEGMENT39_1
        retlw   high SEGMENT39_1
        retlw   low  SEGMENT40_1
        retlw   high SEGMENT40_1
        retlw   low  SEGMENT41_1
        retlw   high SEGMENT41_1
        retlw   low  SEGMENT42_1
        retlw   high SEGMENT42_1
        retlw   low  SEGMENT43_1
        retlw   high SEGMENT43_1
        retlw   low  SEGMENT44_1
        retlw   high SEGMENT44_1
        retlw   low  SEGMENT45_1
        retlw   high SEGMENT45_1
        retlw   low  SEGMENT46_1
        retlw   high SEGMENT46_1
        retlw   low  SEGMENT47_1
        retlw   high SEGMENT47_1
        retlw   low  SEGMENT48_1
        retlw   high SEGMENT48_1
        retlw   low  SEGMENT49_1
        retlw   high SEGMENT49_1
        retlw   low  SEGMENT50_1
        retlw   high SEGMENT50_1
        retlw   low  SEGMENT51_1
        retlw   high SEGMENT51_1
        retlw   low  SEGMENT52_1
        retlw   high SEGMENT52_1
        retlw   low  SEGMENT53_1
        retlw   high SEGMENT53_1
        retlw   low  SEGMENT54_1
        retlw   high SEGMENT54_1
        retlw   low  SEGMENT55_1
        retlw   high SEGMENT55_1
        retlw   low  SEGMENT56_1
        retlw   high SEGMENT56_1
        retlw   low  SEGMENT57_1
        retlw   high SEGMENT57_1
        retlw   low  SEGMENT58_1
        retlw   high SEGMENT58_1
        retlw   low  SEGMENT59_1
        retlw   high SEGMENT59_1
        retlw   low  SEGMENT60_1
        retlw   high SEGMENT60_1
        retlw   low  SEGMENT61_1
        retlw   high SEGMENT61_1
        retlw   low  SEGMENT62_1
        retlw   high SEGMENT62_1
        retlw   low  SEGMENT63_1
        retlw   high SEGMENT63_1
        retlw   low  SEGMENT64_1
        retlw   high SEGMENT64_1


;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
