;----------------------------------------------------------------------
;
;   Filename:           25LC640.asm
;   Microcontroller:    PIC16F684
;   Assembled using:    MPLAB IDE v6.40
;
;   Author:             Steven Bible
;   Company:            Microchip Technology Inc.
;
;
;----------------------------------------------------------------------
;
;   Program Description:
;
;   < type program description here >
;
;----------------------------------------------------------------------
;
;   Revision History:
;
;   Ver. 0.0 - Feburary 7, 2004: Initial write
;
;----------------------------------------------------------------------
; Include Files and Project Files:
;

#include    "project.inc"           ; io port and processor defines
#include    "25lc640.inc"

;----------------------------------------------------------------------
; Externally Defined Labels
;
; EXTERN declares symbol names that are used in this module and declared
; as global in another module.
;----------------------------------------------------------------------

    ; external variables

    

    ; external subroutines

    extern  SPI_PutByte             ; spi.asm
    extern  SPI_GetByte             ; spi.asm
    
;----------------------------------------------------------------------
; Global Defines
;----------------------------------------------------------------------

; see daq1.inc

;----------------------------------------------------------------------
; Local Defines
;----------------------------------------------------------------------

; none

;----------------------------------------------------------------------
; Variables
;----------------------------------------------------------------------

;--------------------
; Uninitialized Data Section
;
; The RES directive reserves space for data storage.

        udata



;--------------------
; Shared Uninitialized Data Section
;
; Variables that are allocated in RAM that are shared across all RAM banks (unbanked RAM). 
; The RES directive reserves space for data storage.
;
        udata_shr

        ; Serial EEPROM

SEE_Adx res 2                       ; Serial EEPROM 16-bit Address variable


;--------------------
; Global Variables
;
; Variables defined in this module that are made available to other 
; modules. The <label> must be defined prior to using the GLOBAL
; directive in this module. 

    global  SEE_Adx


;----------------------------------------------------------------------
; Begin Executable Code Segment
;----------------------------------------------------------------------

;[<label>]   code    [<ROM_Address>]

        code

;----------------------------------------------------------------------
; Subroutine: Serial EEPROM Select Device Macro
;   
; Description: 
;   Select the Serial EEPROM device according bit 2^13 of the address
;   value.
;   
; Constants: 
;   None
;   
; Global Variables: 
;   SEE_Adx = Serial EEPROM address LSB
;   SEE_Adx+1 = Serial EEPROM address MSB
;   
; Initialization: 
;   None
;   
; Output: 
;   None
;   
;----------------------------------------------------------------------

mSEE_Select macro

        btfss   SEE_Adx+1, 5        ; if address bit 2^13 = 0
         bcf    M0_CS               ;   then select Serial EEPROM 0
        btfsc   SEE_Adx+1, 5        ; if address bit 2^13 = 1
         bcf    M1_CS               ;   then select Serial EEPROM 1

        endm


;----------------------------------------------------------------------
; Subroutine: Open Serial EEPROM for Reading
;   
; Description: 
;   Prepare the Serial EEPROM for reading by:
;   1. select the device
;   2. send the 16-bit address
;   This routine must be called prior to reading data from the Serial EEPROM.
;   
; Constants: 
;   
; Global Variables: 
;   SEE_Adx = Serial EEPROM address LSB
;   SEE_Adx+1 = Serial EEPROM address MSB
;   
; Initialization: 
;   1. 
;   
; Output: 
;   
;----------------------------------------------------------------------

SEE_OpenRead
        global  SEE_OpenRead

        ; select the Serial EEPROM

        mSEE_Select

        ; send 8-bit read instruction

        movlw   0x03                ; READ instruction
        call    SPI_PutByte

        ; send 16-bit address

        movfw   SEE_Adx + 1         ; Address MSB
        call    SPI_PutByte

        movfw   SEE_Adx             ; Address LSB
        call    SPI_PutByte

        ; the Serial EEPROM is now ready to read data from using
        ; the SEE_ReadByte subroutine

        return


;----------------------------------------------------------------------
; Subroutine: Read one byte of data from the Serial EEPROM
;   
; Description: 
;
;   The Serial EEPROM must be opened for reading using the SEE_OpenRead
;   subroutine prior to reading a byte.
;   Since there is no page limitations when reading from the Serial
;   EEPROM, this routine can be called continuously and the address
;   pointer will wrap around.
;
; Constants: 
;   None
;   
; Global Variables: 
;   SEE_Adx = Serial EEPROM address LSB
;   SEE_Adx+1 = Serial EEPROM address MSB
;   
; Initialization: 
;   1. SCK should be low prior to entering routine
;   
; Output: 
;   1. The value of the Data is returned in the W register
;   1. SCK will be low at the end of the routine
;   
;----------------------------------------------------------------------

SEE_ReadByte
        global  SEE_ReadByte

        call    SPI_GetByte         ; read Data value

        incfsz  SEE_Adx, F          ; increment the address
         goto   $ + 2
        incf    SEE_Adx+1, F

        return                      ; return the Data value in the W register


;----------------------------------------------------------------------
; Subroutine: Open Serial EEPROM for Writing
;   
; Description: 
;   Prepare the Serial EEPROM for writing by:
;   1. checking to see if a write cycle is in progress (WIP=1)
;   2. select the device
;   3. set the write enable latch
;   4. send WRITE instruction
;   5. send the 16-bit address
;   This routine must be called prior to writing data to the Serial EEPROM.
;   
; Constants: 
;   None
;
; Global Variables: 
;   SEE_Adx = Serial EEPROM address LSB
;   SEE_Adx+1 = Serial EEPROM address MSB
;   
; Initialization: 
;   1. SCK should be low prior to entering routine
;   
; Output: 
;   1. SCK will be low at the end of the routine
;   
;----------------------------------------------------------------------

SEE_OpenWrite
        global  SEE_OpenWrite

        ; read the Status Register

        call    SEE_ReadStatus

        andlw   0x01                ; mask out the WIP bit
        xorlw   0x01                ; is there a write in progress?
        btfsc   STATUS, Z
         goto   SEE_OpenWrite       ; yes, loop until WIP = 0

        ; select the Serial EEPROM

        mSEE_Select

        ; set the write enable latch

        movlw   0x06                ; send 8-bit WREN instruction
        call    SPI_PutByte

        ; deselect the Serial EEPROM (sets the write enable latch)

        call    SEE_Close

        ; select the Serial EEPROM

        mSEE_Select

        ; send 8-bit write instruction

        movlw   0x02                ; WRITE instruction
        call    SPI_PutByte

        ; send 16-bit address

        movfw   SEE_Adx + 1         ; Address MSB
        call    SPI_PutByte

        movfw   SEE_Adx             ; Address LSB
        call    SPI_PutByte

        ; the Serial EEPROM is now ready to write data to using the
        ; SEE_WriteByte subroutine (up to a page of data at a time)

        return


;----------------------------------------------------------------------
; Subroutine: Write one byte of data to the Serial EEPROM
;   
; Description: 
;
;   The Serial EEPROM must be opened for writing data using the 
;   SEE_OpenWrite subroutine.
;   This routine keeps track of the page size and automatically 
;   selects the next page when a page boundary is met.
;   
; Constants: 
;   SEE_PageMask = Serial EEPROM Page Mask value
;   
; Global Variables: 
;   SEE_Adx = Serial EEPROM address LSB
;   SEE_Adx+1 = Serial EEPROM address MSB
;   
; Initialization: 
;   1. SCK should be low prior to entering routine
;   2. W register contains the 8-bit data byte to write
;   
; Output: 
;   1. SCK will be low at the end of the routine
;   
;----------------------------------------------------------------------

SEE_WriteByte
        global  SEE_WriteByte

        call    SPI_PutByte

        movfw   SEE_Adx             ; get Address LSB
        andlw   SEE_PageMask        ; mask all address bits in the page
        xorlw   SEE_PageMask        ; are all page bits set?
        btfsc   STATUS, Z
         call   SEE_Close           ; yes, we are on the page boundary
                                    ;   close the current page and
                                    ;   deselect the Serial EEPROM to write data

        incfsz  SEE_Adx, F          ; no, increment the address
         goto   $ + 2
        incf    SEE_Adx+1, F

        movfw   SEE_Adx             ; get Address LSB
        andlw   SEE_PageMask        ; mask all address bits in the page
        btfsc   STATUS, Z           ; if page = 0,
         call   SEE_OpenWrite       ;   open a new page for writing

        return


;----------------------------------------------------------------------
; Subroutine: Close the Serial EEPROM for Reading or Writing
;   
; Description: 
;   Closes the current page by deselecting the Serial EEPROM. If data
;   has been written to the Serial EEPROM, this begins the write cycle 
;   which can take a maximum of 5 ms to complete.   
;   
; Constants: 
;   None
;   
; Global Variables: 
;   None
;   
; Initialization: 
;   None
;   
; Output: 
;   None
;   
;----------------------------------------------------------------------

SEE_Close
        global  SEE_Close

        bsf     M0_CS               ; deselect Serial EEPROM 0
        bsf     M1_CS               ; deselect Serial EEPROM 1

        return


;----------------------------------------------------------------------
; Subroutine: Read Serial EEPROM Status Register
;   
; Description: 
;
;   Read the Serial EEPROM Status Register. The Status Register can be
;   read at any time, even during a write cycle. The value of the Status 
;   Register is returned in the W register.
;   
; Constants: 
;   None
;   
; Global Variables: 
;   SEE_Adx = Serial EEPROM address LSB
;   SEE_Adx+1 = Serial EEPROM address MSB
;   
; Initialization: 
;   1. SCK should be low prior to entering this routine
;   2. W register contains the 8-bit byte to write
;   
; Output: 
;   1. The value of the Status Register is returned in the W register
;   2. SCK will be low at the end of the routine
;   
;----------------------------------------------------------------------

SEE_ReadStatus
        global  SEE_ReadStatus

        ; select Serial EEPROM

        mSEE_Select

        ; send 8-bit read status register instruction

        movlw   0x05                ; RDSR instruction
        call    SPI_PutByte

        call    SPI_GetByte         ; Get the Status Register value

        call    SEE_Close

        return                      ; return Status Register value in W register


;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
