
; Header File project.inc

;----------------------------------------------------------------------
; I/O Port and Project Defines
; Microcontroller:    PIC16F684
;----------------------------------------------------------------------

#include    <p16f684.inc>           ; processor specific variable definitions

;----------------------------------------------------------------------
; PORT Defines
;----------------------------------------------------------------------
    ;--------------------
    ; PORTA (Section 4.1)
    ;--------------------
    ; PORTA is an 6-bit wide, bi-directional port. The corresponding data
    ; direction register is TRISA. Setting a TRISA bit (= 1) will make
    ; the corresponding PORTA pin and input. Clearing a TRISA bit (= 0)
    ; will make the corresponding PORTA pin an output. The exception is
    ; RA3, which is input only and its TRIS bit will always read as a '1'.
    ;
    ; Function of PORTA pins depend on:
    ;   Configuration Bits (CONFIG) (Section 12.1)
    ;   Weak Pull-up Register (WPUA) (Section 4.2.1)
    ;   Interrupt-on-change Register (IOCB) (Section 4.2.2)
    ;   Ultra-low Power Wake-up (ULPWU) (Section 4.2.3)
    ;   Option Register (OPTION_REG) (Register 2-1)
    ;   TIMER1 Control Register (T1CON) (Register 6-1)
    ;   Comparator Control Register (CMCON) (Section 9.0)
    ;   A/D Control Register (ADCON0) (Section 10.0)

#define SSCDAT      PORTA, 0        ; (Digital Input/Output) Synchronous Serial Data (bidirectional I/O to PICkit 1) 
#define SSCCLK      PORTA, 1        ; (Digital Input/Output) Synchronous Serial Clock (bidirectional I/O to PICkit 1)
#define MISO        PORTA, 2        ; (Digital Input) SPI Master In, Slave Out
#define RA3         PORTA, 3        ; (Digital Input Only) RA3/Vpp
#define M1_CS       PORTA, 4        ; (Digital Output) Memory 1 Chip Select
#define M0_CS       PORTA, 5        ; (Digital Output) Memory 0 Chip Select

    ; Define for TRISA Register (Section 4.1)

    ;   PORTA Pins = xx543210
#define PORTATRIS  b'00001111'

    ;--------------------
    ; PORTC (Section 3.3)
    ; --------------------
    ; PORTC is a general purpose I/O port consisting of 6 bi-directional
    ; pins. The pins can be configured for either digital I/O or analog
    ; input to A/D converter. For specific information about individual
    ; functions such as the comparator or the A/D, refer to the 
    ; appropriate section in the data sheet.
    ;
    ;   Comparator Module (Section 9.0)
    ;   Analog-to-Digital Converter (A/D) Module (Section 10.0)

#define PGA_Vout    PORTC, 0        ; (Analog Input) PGA Vout (AN4)
#define RC1         PORTC, 1        ; (Input/Output) 
#define MOSI        PORTC, 2        ; (Digital Output) SPI Master Out, Slave In
#define PGA_CS      PORTC, 3        ; (Digital Output) PGA Chip Select (active low)
#define SCK         PORTC, 4        ; (Digital Output) SPI Serial Clock
#define RC5         PORTC, 5        ; (Input/Output) 

    ; Define for TRISC Register (Section 4.3)

    ;   PORTC Pins = xx543210
#define PORTCTRIS  b'00100011'

;----------------------------------------------------------------------
; Global Defines for Project
;----------------------------------------------------------------------

#define CLOCKSPEED  .8000000        ; 8MHz internal Osc
;#define CLOCKSPEED  .4000000        ; 4MHz internal Osc

#define VERSION     0x10            ; Firmware version number (Major:Minor)

#define CMD_UNSUCCESSFUL            0xFF
#define CMD_Version_SUCCESSFUL      0x10  ; Version command was successful
#define CMD_0x11_SUCCESSFUL         0x11  ; 
#define CMD_0x12_SUCCESSFUL         0x12  ; 
#define CMD_0x13_SUCCESSFUL         0x13  ; 
#define CMD_0x14_SUCCESSFUL         0x14  ; 
#define CMD_0x15_SUCCESSFUL         0x15  ; 
#define CMD_0x16_SUCCESSFUL         0x16  ; 
