;----------------------------------------------------------------------
;
;   Filename:           sssc.asm
;   Microcontroller:    
;   Assembled using:    MPLAB IDE v6.40
;
;   Author:             Steven Bible
;   Company:            Microchip Technology Inc.
;
;
;----------------------------------------------------------------------
;
;   Program Description:
;
;   < type program description here >
;
;----------------------------------------------------------------------
;
;   Revision History:
;
;   Ver. 0.0 - February 7, 2004: Initial write
;
;----------------------------------------------------------------------
; Include Files and Project Files:
;

#include    "project.inc"           ; io port and processor defines

;----------------------------------------------------------------------
; External Defines
;----------------------------------------------------------------------

    ; Place #define directives in the project_io_ports.inc file

;#define SSCDAT      PORTA, 0
;#define SSCCLK      PORTA, 1

;#define CLOCKSPEED      .20000000	; 20MHz internal Osc
;#define CLOCKSPEED      .8000000	; 8MHz internal Osc
;#define CLOCKSPEED      .4000000	; 4MHz internal Osc


;----------------------------------------------------------------------
; Global Defines
;----------------------------------------------------------------------

; see daq1.inc

;----------------------------------------------------------------------
; Local Defines
;----------------------------------------------------------------------

    ; None

;----------------------------------------------------------------------
; Variables
;----------------------------------------------------------------------

;--------------------
; Uninitialized Data Section
;
; The RES directive should be used to reserve space for data storage.

    ;udata



;--------------------
; Shared Uninitialized Data Section
;
; Variables that are allocated in RAM that are shared across all RAM banks (unbanked RAM). 
; The RES directive should be used to reserve space for data storage.
;
    udata_shr

TxByte              res 1
BitCounter          res 1
DelayCounter        res 1
SSC_RxByte          res 1


;--------------------
; Global Variables
;
; Variables defined in this module that should be available to other
; modules. The <label> must be defined prior to using the GLOBAL directive. 

    global  SSC_RxByte

;----------------------------------------------------------------------
; Begin Executable Code Segment
;----------------------------------------------------------------------

;[<label>]  code    [<ROM_Address>]

            code

; *********************************************************************
; Reads a byte from the Synchronous Serial link (Slave mode?)
; Returns byte in W when the databyte is complete
; *********************************************************************
SSC_GetByte
        global  SSC_GetByte

        clrf    SSC_RxByte          ; clear the received byte variable

        movlw   .8                  ; load bit counter with 8
        movwf   BitCounter

        btfsc   SSCCLK              ; Is the clock low?
        goto    $-1                 ;   no, keep waiting

GBLoop
        btfss   SSCCLK              ; Is the clock high?
        goto    $-1                 ;   yes, go read the incoming byte

        bcf     STATUS, C           ; clear carry now to save the cycle after falling edge

        btfsc   SSCCLK              ; waiting for low, is it?
        goto    $-1

        btfsc   SSCDAT              ; We've already assumed the bit is a 0 is it?
        bsf     STATUS, C           ; no, set carry
        rrf     SSC_RxByte, F       ; rotate carry into byte

        decfsz  BitCounter, F       ; has the entire byte been sent?
         goto   GBLoop              ; no, loop

        movfw   SSC_RxByte          ; return received byte in W register

        return


; *********************************************************************
; SendByte
; Sends a byte in Synchronous Master Mode.  
; Master reads it on the falling edge.
; *********************************************************************
SSC_SendByte
        global  SSC_SendByte

        movwf   TxByte              ; save byte to send

        bsf     SSCCLK              ; set CLK high

        bsf	    STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     SSCCLK              ; set TRIS bit to output
        bcf     SSCDAT              ; set TRIS bit to output

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        movlw   .8                  ; load bit counter with 8
        movwf   BitCounter

SCBSLoop
        bsf     SSCCLK              ; set CLK high

        call    HalfBitTimeDelay

        btfsc   TxByte, 0           ; is bit 0 high?
         bsf    SSCDAT              ;   yes, set Data pin high

        btfss   TxByte, 0           ;   no, set Data pin low
         bcf    SSCDAT

        rrf     TxByte, F

        call    HalfBitTimeDelay

        bcf     SSCCLK              ; set CLK low

        call    BitTimeDelay

        decfsz  BitCounter, F       ; has the entire byte been sent?
         goto   SCBSLoop            ; no, loop
			
;EndSCBS
        call    BitTimeDelay        ; add in extra delay after

        bcf     SSCCLK              ; set CLK low
        bcf     SSCDAT              ; set Data pin low

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bsf     SSCCLK              ; set TRIS bit to input
        bsf     SSCDAT              ; set TRIS bit to input

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        return


; *********************************************************************
;
; *********************************************************************
HalfBitTimeDelay

        if CLOCKSPEED == .4000000
            movlw	.4	;  4 Mhz delay
        else
            if CLOCKSPEED == .8000000
                movlw	.8	;  8 Mhz delay
            else
                if CLOCKSPEED == .20000000
                    movlw	.6	; 20 Mhz delay
                else 
                    messg "HalfBitTime delay not configured for clockspeed"
                endif
            endif
        endif

        goto	TimeDelay

BitTimeDelay

        if CLOCKSPEED == .4000000
            movlw   .7                      ; 4 Mhz delay
        else
            if CLOCKSPEED == .8000000
                movlw   .16                 ; 8 Mhz delay
            else
                if CLOCKSPEED == .20000000
                    movlw   .12             ; 20 Mhz delay
                else 
                    messg "HalfBitTime delay not configured for clockspeed"
                endif
            endif
        endif

TimeDelay

        movwf   DelayCounter
        decfsz  DelayCounter, F
         goto   $-1

        return


;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
