;----------------------------------------------------------------------
;
;   Filename:           therm design 2.asm
;   Microcontroller:    PIC16F684
;   Assembled using:    MPLAB IDE v6.40
;
;   Author:             John Day
;   Company:            Microchip Technology Inc.
;
;----------------------------------------------------------------------
;
;   Program Description:
;
;   < type program description here >
;
;----------------------------------------------------------------------
;
;   Revision History:
;
;   Ver. 0.0 - March 18, 2004: Initial write
;   Ver. 0.1 - August 26, 2004: ported to themistor pga PICtail demonstration
;
;----------------------------------------------------------------------
; Include Files and Project Files:
;

#include    "project.inc"           ; io port and processor defines
#include    "therm design 2.inc"
#include    "pga.inc"

;----------------------------------------------------------------------
; Externally Defined Labels
;
; EXTERN declares symbol names that are used in this module and declared
; as global in another module.
;----------------------------------------------------------------------

    ; external variables



    ; external subroutines



;----------------------------------------------------------------------
; Local Defines
;----------------------------------------------------------------------

    ; none

;----------------------------------------------------------------------
; Variables
;----------------------------------------------------------------------

;--------------------
; Uninitialized Data Section
;
; The RES directive reserves space for data storage.

        udata

Therm_PwLI_IN   res 2               ; Input to PwLI subroutine
Therm_PwLI_OUT  res 2               ; 16-bit output from PwLI subroutine

PwLI_IDX        res 2               ; Index into PwLI table
PwLI_SL         res 3               ; 24-bit slope variable
PwLI_MUL        res 3               ; 24-bit multiply result variable

Temp            res 1               ; temporary variable

PwLITABLE       res 2               ; PwLI Table variable
PGAgain         res 1               ; PGA gain variable

;--------------------
; Shared Uninitialized Data Section
;
; Variables that are allocated in RAM that ais shared across all RAM banks (unbanked RAM). 
; The RES directive reserves space for data storage.
;
; PIC16F684 has 16 shared RAM locations
;
;        udata_shr



;--------------------
; Global Variables
;
; Variables defined in this module that are made available to other 
; modules. The <label> must be defined prior to using the GLOBAL
; directive in this module. 

    global  Therm_PwLI_IN
    global  Therm_PwLI_OUT

;----------------------------------------------------------------------
; Begin Executable Code Segment
;----------------------------------------------------------------------

;[<label>]   code    [<ROM_Address>]

        code

;----------------------------------------------------------------------
; Macros:
;----------------------------------------------------------------------

m_add24 macro   input, output       ; output[3] = input[3] + output[3]

        movf    input, W            ; Add input LSB to output LSB
        addwf   output, F
        movf    input+1, W
        btfsc   STATUS, C
        incfsz  input+1, W          ; Add one if there was a carry from previous add
        addwf   output+1, F         ; Add input+1 to output+1
        movf    input+2, W
        btfsc   STATUS, C
        incfsz  input+2, W          ; Add one if there was a carry from previous add
        addwf   output+2, F         ; Add input+2 to output+2

    endm

;----------------------------------------------------------------------
; Subroutine: Piecewise-Linear Interpolation
;   
; Description: 10-bit input, 16-bit output, 64 PL segments, 6 PL bits
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: 
;   
; Output: 
;   
;----------------------------------------------------------------------

Therm_PwLI
        global  Therm_PwLI

        movwf   PGAgain

        ; calculate PwLI table index (PwLI_IDX) ( 10-bit INPUT; 6-bit PL_TABLE entry )
        ; right shift 3 places (PWLINDEX is multiplied by 2)

        movfw   Therm_PwLI_IN+1
        movwf   PwLI_IDX+1

        movfw   Therm_PwLI_IN
        movwf   PwLI_IDX

        bcf     STATUS, C
        rrf     PwLI_IDX+1, F
        rrf     PwLI_IDX, F

        bcf     STATUS, C
        rrf     PwLI_IDX+1, F
        rrf     PwLI_IDX, F

        bcf     STATUS, C
        rrf     PwLI_IDX+1, F
        rrf     PwLI_IDX, F

        ; at this point, PWLINDEX should contain only 6 bits in positions 6:1

        movlw   0x7E
        andwf   PwLI_IDX,F

; get Y1

        movfw   PwLI_IDX
        call    PwLI_TABLE
        movwf   Therm_PwLI_OUT

        movfw   PwLI_IDX
        addlw   1
        call    PwLI_TABLE
        movwf   Therm_PwLI_OUT+1

; get Y2

        movfw   PwLI_IDX
        addlw   2				
        call    PwLI_TABLE
        movwf   PwLI_SL

        movfw   PwLI_IDX
        addlw   3				
        call    PwLI_TABLE
        movwf   PwLI_SL+1

; calculate intermediate slope value of segment
; subtract Y1 from Y2 (rise within segment), store result in SLOPE

        movfw   Therm_PwLI_OUT
        subwf   PwLI_SL, F
        movf    Therm_PwLI_OUT+1, W
        btfss   STATUS, C
         incfsz Therm_PwLI_OUT+1, W
        subwf   PwLI_SL+1, F
        clrf    PwLI_SL+2
        btfss   STATUS, C
         comf   PwLI_SL+2, F

; calculate Y within segment

        ; multiply lower 4 bits of PwLI_IN times intermediate SLOPE (the full RISE (or FALL) of the segment)

        clrf    PwLI_MUL            ; Clear multiplication result register
        clrf    PwLI_MUL+1
        clrf    PwLI_MUL+2

        btfss   Therm_PwLI_IN, 0      ; Check bit 0 of multiplicand, skip add if it is zero
         goto   skip_add_4
        m_add24 PwLI_SL, PwLI_MUL     ; Add multiplier to result if bit 0 of multiplicand is 1

skip_add_4
        bcf     STATUS, C           ; Clear carry bit in anticipation of a 24-bit rotate
        rlf     PwLI_SL, F            ; Rotate SLOPE[3] one position to the left (multiply by 2)
        rlf     PwLI_SL+1, F
        rlf     PwLI_SL+2, F
        btfss   Therm_PwLI_IN, 1      ; Check bit 1 of multiplicand, skip add if it is zero
         goto   skip_add_3
        m_add24 PwLI_SL, PwLI_MUL    ; Add multiplier to result if bit 1 of multiplicand is 1

skip_add_3	
        rlf     PwLI_SL, F            ; Rotate SLOPE[3] one position to the left (multiply by 2)
        rlf     PwLI_SL+1, F
        rlf     PwLI_SL+2, F
        btfss   Therm_PwLI_IN, 2      ; Check bit 2 of multiplicand, skip add if it is zero
         goto   skip_add_2
        m_add24 PwLI_SL, PwLI_MUL    ; Add multiplier to result if bit 2 of multiplicand is 1

skip_add_2	
        rlf     PwLI_SL, F            ; Rotate SLOPE[3] one position to the left (multiply by 2)
        rlf     PwLI_SL+1, F
        rlf     PwLI_SL+2, F
        btfss   Therm_PwLI_IN, 3      ; Check bit 3 of multiplicand, skip add if it is zero
         goto   skip_add_1
        m_add24 PwLI_SL, PwLI_MUL    ; Add multiplier to result if bit 3 of multiplicand is 1

        ; divide by 16 (now we have Y)

skip_add_1
        swapf   PwLI_MUL, W
        andlw   0x0F
        movwf   PwLI_MUL
        swapf   PwLI_MUL+1, W
        andlw   0xF0
        iorwf   PwLI_MUL, F
        swapf   PwLI_MUL+1, W
        andlw   0x0F
        movwf   PwLI_MUL+1
        swapf   PwLI_MUL+2, W
        andlw   0xF0
        iorwf   PwLI_MUL+1, F

; add Y of the segment to Y1, result in PwLI_OUT
		
        movfw   PwLI_MUL
        addwf   Therm_PwLI_OUT, F
        movfw   PwLI_MUL+1
        btfsc   STATUS, C
         addlw  1
        addwf   Therm_PwLI_OUT+1, F

        return


;----------------------------------------
; Piecewise Linear Interpolation Look-up table

PwLI_TABLE

        movwf   Temp                ; save W (index into table)

        movfw   PGAgain             ; if (PGAgain = 1) {
        xorlw   PGA_GAIN_1
        btfss   STATUS, Z
         goto   PT_8

        movlw   HIGH PwLITABLE_1    ;   PwLITABLE = PwLITABLE_1
        movwf   PwLITABLE
        movlw   LOW PwLITABLE_1
        movwf   PwLITABLE+1

        goto    PT_End
PT_8
        movfw   PGAgain             ; if (PGAgain = 8) {
        xorlw   PGA_GAIN_8
        btfss   STATUS, Z
         goto   PT_32

        movlw   HIGH PwLITABLE_8    ;   PwLITABLE = PwLITABLE_8
        movwf   PwLITABLE
        movlw   LOW PwLITABLE_8
        movwf   PwLITABLE+1

        goto    PT_End
PT_32
        movfw   PGAgain             ; if (PGAgain = 32) {
        xorlw   PGA_GAIN_32
        btfss   STATUS, Z
         goto   PT_End

        movlw   HIGH PwLITABLE_32   ;   PwLITABLE = PwLITABLE_32
        movwf   PwLITABLE
        movlw   LOW PwLITABLE_32
        movwf   PwLITABLE+1
PT_End
        movfw   PwLITABLE
        movwf   PCLATH
        movfw   Temp
        addwf   PwLITABLE+1, W
        btfsc   STATUS, C
         incf   PCLATH, F
        movwf   PCL

PwLITABLE_1
        retlw   low  SEGMENT0_1
        retlw	high SEGMENT0_1
        retlw   low  SEGMENT1_1
        retlw   high SEGMENT1_1
        retlw   low  SEGMENT2_1
        retlw   high SEGMENT2_1
        retlw   low  SEGMENT3_1
        retlw   high SEGMENT3_1
        retlw   low  SEGMENT4_1
        retlw   high SEGMENT4_1
        retlw   low  SEGMENT5_1
        retlw   high SEGMENT5_1
        retlw   low  SEGMENT6_1
        retlw   high SEGMENT6_1
        retlw   low  SEGMENT7_1
        retlw   high SEGMENT7_1
        retlw   low  SEGMENT8_1
        retlw   high SEGMENT8_1
        retlw   low  SEGMENT9_1
        retlw   high SEGMENT9_1
        retlw   low  SEGMENT10_1
        retlw   high SEGMENT10_1
        retlw   low  SEGMENT11_1
        retlw   high SEGMENT11_1
        retlw   low  SEGMENT12_1
        retlw   high SEGMENT12_1
        retlw   low  SEGMENT13_1
        retlw   high SEGMENT13_1
        retlw   low  SEGMENT14_1
        retlw   high SEGMENT14_1
        retlw   low  SEGMENT15_1
        retlw   high SEGMENT15_1
        retlw   low  SEGMENT16_1
        retlw   high SEGMENT16_1
        retlw   low  SEGMENT17_1
        retlw   high SEGMENT17_1
        retlw   low  SEGMENT18_1
        retlw   high SEGMENT18_1
        retlw   low  SEGMENT19_1
        retlw   high SEGMENT19_1
        retlw   low  SEGMENT20_1
        retlw   high SEGMENT20_1
        retlw   low  SEGMENT21_1
        retlw   high SEGMENT21_1
        retlw   low  SEGMENT22_1
        retlw   high SEGMENT22_1
        retlw   low  SEGMENT23_1
        retlw   high SEGMENT23_1
        retlw   low  SEGMENT24_1
        retlw   high SEGMENT24_1
        retlw   low  SEGMENT25_1
        retlw   high SEGMENT25_1
        retlw   low  SEGMENT26_1
        retlw   high SEGMENT26_1
        retlw   low  SEGMENT27_1
        retlw   high SEGMENT27_1
        retlw   low  SEGMENT28_1
        retlw   high SEGMENT28_1
        retlw   low  SEGMENT29_1
        retlw   high SEGMENT29_1
        retlw   low  SEGMENT30_1
        retlw   high SEGMENT30_1
        retlw   low  SEGMENT31_1
        retlw   high SEGMENT31_1
        retlw   low  SEGMENT32_1
        retlw   high SEGMENT32_1
        retlw   low  SEGMENT33_1
        retlw   high SEGMENT33_1
        retlw   low  SEGMENT34_1
        retlw   high SEGMENT34_1
        retlw   low  SEGMENT35_1
        retlw   high SEGMENT35_1
        retlw   low  SEGMENT36_1
        retlw   high SEGMENT36_1
        retlw   low  SEGMENT37_1
        retlw   high SEGMENT37_1
        retlw   low  SEGMENT38_1
        retlw   high SEGMENT38_1
        retlw   low  SEGMENT39_1
        retlw   high SEGMENT39_1
        retlw   low  SEGMENT40_1
        retlw   high SEGMENT40_1
        retlw   low  SEGMENT41_1
        retlw   high SEGMENT41_1
        retlw   low  SEGMENT42_1
        retlw   high SEGMENT42_1
        retlw   low  SEGMENT43_1
        retlw   high SEGMENT43_1
        retlw   low  SEGMENT44_1
        retlw   high SEGMENT44_1
        retlw   low  SEGMENT45_1
        retlw   high SEGMENT45_1
        retlw   low  SEGMENT46_1
        retlw   high SEGMENT46_1
        retlw   low  SEGMENT47_1
        retlw   high SEGMENT47_1
        retlw   low  SEGMENT48_1
        retlw   high SEGMENT48_1
        retlw   low  SEGMENT49_1
        retlw   high SEGMENT49_1
        retlw   low  SEGMENT50_1
        retlw   high SEGMENT50_1
        retlw   low  SEGMENT51_1
        retlw   high SEGMENT51_1
        retlw   low  SEGMENT52_1
        retlw   high SEGMENT52_1
        retlw   low  SEGMENT53_1
        retlw   high SEGMENT53_1
        retlw   low  SEGMENT54_1
        retlw   high SEGMENT54_1
        retlw   low  SEGMENT55_1
        retlw   high SEGMENT55_1
        retlw   low  SEGMENT56_1
        retlw   high SEGMENT56_1
        retlw   low  SEGMENT57_1
        retlw   high SEGMENT57_1
        retlw   low  SEGMENT58_1
        retlw   high SEGMENT58_1
        retlw   low  SEGMENT59_1
        retlw   high SEGMENT59_1
        retlw   low  SEGMENT60_1
        retlw   high SEGMENT60_1
        retlw   low  SEGMENT61_1
        retlw   high SEGMENT61_1
        retlw   low  SEGMENT62_1
        retlw   high SEGMENT62_1
        retlw   low  SEGMENT63_1
        retlw   high SEGMENT63_1
        retlw   low  SEGMENT64_1
        retlw   high SEGMENT64_1

PwLITABLE_8
        retlw   low  SEGMENT0_8
        retlw	high SEGMENT0_8
        retlw   low  SEGMENT1_8
        retlw   high SEGMENT1_8
        retlw   low  SEGMENT2_8
        retlw   high SEGMENT2_8
        retlw   low  SEGMENT3_8
        retlw   high SEGMENT3_8
        retlw   low  SEGMENT4_8
        retlw   high SEGMENT4_8
        retlw   low  SEGMENT5_8
        retlw   high SEGMENT5_8
        retlw   low  SEGMENT6_8
        retlw   high SEGMENT6_8
        retlw   low  SEGMENT7_8
        retlw   high SEGMENT7_8
        retlw   low  SEGMENT8_8
        retlw   high SEGMENT8_8
        retlw   low  SEGMENT9_8
        retlw   high SEGMENT9_8
        retlw   low  SEGMENT10_8
        retlw   high SEGMENT10_8
        retlw   low  SEGMENT11_8
        retlw   high SEGMENT11_8
        retlw   low  SEGMENT12_8
        retlw   high SEGMENT12_8
        retlw   low  SEGMENT13_8
        retlw   high SEGMENT13_8
        retlw   low  SEGMENT14_8
        retlw   high SEGMENT14_8
        retlw   low  SEGMENT15_8
        retlw   high SEGMENT15_8
        retlw   low  SEGMENT16_8
        retlw   high SEGMENT16_8
        retlw   low  SEGMENT17_8
        retlw   high SEGMENT17_8
        retlw   low  SEGMENT18_8
        retlw   high SEGMENT18_8
        retlw   low  SEGMENT19_8
        retlw   high SEGMENT19_8
        retlw   low  SEGMENT20_8
        retlw   high SEGMENT20_8
        retlw   low  SEGMENT21_8
        retlw   high SEGMENT21_8
        retlw   low  SEGMENT22_8
        retlw   high SEGMENT22_8
        retlw   low  SEGMENT23_8
        retlw   high SEGMENT23_8
        retlw   low  SEGMENT24_8
        retlw   high SEGMENT24_8
        retlw   low  SEGMENT25_8
        retlw   high SEGMENT25_8
        retlw   low  SEGMENT26_8
        retlw   high SEGMENT26_8
        retlw   low  SEGMENT27_8
        retlw   high SEGMENT27_8
        retlw   low  SEGMENT28_8
        retlw   high SEGMENT28_8
        retlw   low  SEGMENT29_8
        retlw   high SEGMENT29_8
        retlw   low  SEGMENT30_8
        retlw   high SEGMENT30_8
        retlw   low  SEGMENT31_8
        retlw   high SEGMENT31_8
        retlw   low  SEGMENT32_8
        retlw   high SEGMENT32_8
        retlw   low  SEGMENT33_8
        retlw   high SEGMENT33_8
        retlw   low  SEGMENT34_8
        retlw   high SEGMENT34_8
        retlw   low  SEGMENT35_8
        retlw   high SEGMENT35_8
        retlw   low  SEGMENT36_8
        retlw   high SEGMENT36_8
        retlw   low  SEGMENT37_8
        retlw   high SEGMENT37_8
        retlw   low  SEGMENT38_8
        retlw   high SEGMENT38_8
        retlw   low  SEGMENT39_8
        retlw   high SEGMENT39_8
        retlw   low  SEGMENT40_8
        retlw   high SEGMENT40_8
        retlw   low  SEGMENT41_8
        retlw   high SEGMENT41_8
        retlw   low  SEGMENT42_8
        retlw   high SEGMENT42_8
        retlw   low  SEGMENT43_8
        retlw   high SEGMENT43_8
        retlw   low  SEGMENT44_8
        retlw   high SEGMENT44_8
        retlw   low  SEGMENT45_8
        retlw   high SEGMENT45_8
        retlw   low  SEGMENT46_8
        retlw   high SEGMENT46_8
        retlw   low  SEGMENT47_8
        retlw   high SEGMENT47_8
        retlw   low  SEGMENT48_8
        retlw   high SEGMENT48_8
        retlw   low  SEGMENT49_8
        retlw   high SEGMENT49_8
        retlw   low  SEGMENT50_8
        retlw   high SEGMENT50_8
        retlw   low  SEGMENT51_8
        retlw   high SEGMENT51_8
        retlw   low  SEGMENT52_8
        retlw   high SEGMENT52_8
        retlw   low  SEGMENT53_8
        retlw   high SEGMENT53_8
        retlw   low  SEGMENT54_8
        retlw   high SEGMENT54_8
        retlw   low  SEGMENT55_8
        retlw   high SEGMENT55_8
        retlw   low  SEGMENT56_8
        retlw   high SEGMENT56_8
        retlw   low  SEGMENT57_8
        retlw   high SEGMENT57_8
        retlw   low  SEGMENT58_8
        retlw   high SEGMENT58_8
        retlw   low  SEGMENT59_8
        retlw   high SEGMENT59_8
        retlw   low  SEGMENT60_8
        retlw   high SEGMENT60_8
        retlw   low  SEGMENT61_8
        retlw   high SEGMENT61_8
        retlw   low  SEGMENT62_8
        retlw   high SEGMENT62_8
        retlw   low  SEGMENT63_8
        retlw   high SEGMENT63_8
        retlw   low  SEGMENT64_8
        retlw   high SEGMENT64_8

PwLITABLE_32
        retlw   low  SEGMENT0_32
        retlw	high SEGMENT0_32
        retlw   low  SEGMENT1_32
        retlw   high SEGMENT1_32
        retlw   low  SEGMENT2_32
        retlw   high SEGMENT2_32
        retlw   low  SEGMENT3_32
        retlw   high SEGMENT3_32
        retlw   low  SEGMENT4_32
        retlw   high SEGMENT4_32
        retlw   low  SEGMENT5_32
        retlw   high SEGMENT5_32
        retlw   low  SEGMENT6_32
        retlw   high SEGMENT6_32
        retlw   low  SEGMENT7_32
        retlw   high SEGMENT7_32
        retlw   low  SEGMENT8_32
        retlw   high SEGMENT8_32
        retlw   low  SEGMENT9_32
        retlw   high SEGMENT9_32
        retlw   low  SEGMENT10_32
        retlw   high SEGMENT10_32
        retlw   low  SEGMENT11_32
        retlw   high SEGMENT11_32
        retlw   low  SEGMENT12_32
        retlw   high SEGMENT12_32
        retlw   low  SEGMENT13_32
        retlw   high SEGMENT13_32
        retlw   low  SEGMENT14_32
        retlw   high SEGMENT14_32
        retlw   low  SEGMENT15_32
        retlw   high SEGMENT15_32
        retlw   low  SEGMENT16_32
        retlw   high SEGMENT16_32
        retlw   low  SEGMENT17_32
        retlw   high SEGMENT17_32
        retlw   low  SEGMENT18_32
        retlw   high SEGMENT18_32
        retlw   low  SEGMENT19_32
        retlw   high SEGMENT19_32
        retlw   low  SEGMENT20_32
        retlw   high SEGMENT20_32
        retlw   low  SEGMENT21_32
        retlw   high SEGMENT21_32
        retlw   low  SEGMENT22_32
        retlw   high SEGMENT22_32
        retlw   low  SEGMENT23_32
        retlw   high SEGMENT23_32
        retlw   low  SEGMENT24_32
        retlw   high SEGMENT24_32
        retlw   low  SEGMENT25_32
        retlw   high SEGMENT25_32
        retlw   low  SEGMENT26_32
        retlw   high SEGMENT26_32
        retlw   low  SEGMENT27_32
        retlw   high SEGMENT27_32
        retlw   low  SEGMENT28_32
        retlw   high SEGMENT28_32
        retlw   low  SEGMENT29_32
        retlw   high SEGMENT29_32
        retlw   low  SEGMENT30_32
        retlw   high SEGMENT30_32
        retlw   low  SEGMENT31_32
        retlw   high SEGMENT31_32
        retlw   low  SEGMENT32_32
        retlw   high SEGMENT32_32
        retlw   low  SEGMENT33_32
        retlw   high SEGMENT33_32
        retlw   low  SEGMENT34_32
        retlw   high SEGMENT34_32
        retlw   low  SEGMENT35_32
        retlw   high SEGMENT35_32
        retlw   low  SEGMENT36_32
        retlw   high SEGMENT36_32
        retlw   low  SEGMENT37_32
        retlw   high SEGMENT37_32
        retlw   low  SEGMENT38_32
        retlw   high SEGMENT38_32
        retlw   low  SEGMENT39_32
        retlw   high SEGMENT39_32
        retlw   low  SEGMENT40_32
        retlw   high SEGMENT40_32
        retlw   low  SEGMENT41_32
        retlw   high SEGMENT41_32
        retlw   low  SEGMENT42_32
        retlw   high SEGMENT42_32
        retlw   low  SEGMENT43_32
        retlw   high SEGMENT43_32
        retlw   low  SEGMENT44_32
        retlw   high SEGMENT44_32
        retlw   low  SEGMENT45_32
        retlw   high SEGMENT45_32
        retlw   low  SEGMENT46_32
        retlw   high SEGMENT46_32
        retlw   low  SEGMENT47_32
        retlw   high SEGMENT47_32
        retlw   low  SEGMENT48_32
        retlw   high SEGMENT48_32
        retlw   low  SEGMENT49_32
        retlw   high SEGMENT49_32
        retlw   low  SEGMENT50_32
        retlw   high SEGMENT50_32
        retlw   low  SEGMENT51_32
        retlw   high SEGMENT51_32
        retlw   low  SEGMENT52_32
        retlw   high SEGMENT52_32
        retlw   low  SEGMENT53_32
        retlw   high SEGMENT53_32
        retlw   low  SEGMENT54_32
        retlw   high SEGMENT54_32
        retlw   low  SEGMENT55_32
        retlw   high SEGMENT55_32
        retlw   low  SEGMENT56_32
        retlw   high SEGMENT56_32
        retlw   low  SEGMENT57_32
        retlw   high SEGMENT57_32
        retlw   low  SEGMENT58_32
        retlw   high SEGMENT58_32
        retlw   low  SEGMENT59_32
        retlw   high SEGMENT59_32
        retlw   low  SEGMENT60_32
        retlw   high SEGMENT60_32
        retlw   low  SEGMENT61_32
        retlw   high SEGMENT61_32
        retlw   low  SEGMENT62_32
        retlw   high SEGMENT62_32
        retlw   low  SEGMENT63_32
        retlw   high SEGMENT63_32
        retlw   low  SEGMENT64_32
        retlw   high SEGMENT64_32


;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
