;----------------------------------------------------------------------
; 
; Copyright 2005 Microchip Technology Inc.
; 
; Microchip Technology Inc. ("Microchip") licenses this software to
; you solely for use with Microchip products. The software is owned
; by Microchip and is protected under applicable copyright laws. All
; rights reserved.
; 
; SOFTWARE IS PROVIDED IN AN "AS IS." MICROCHIP EXPRESSLY DISCLAIMS ANY
; WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
; PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
; BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
; DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
; PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
; BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
; ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
; 
;----------------------------------------------------------------------
;
;   Filename:           initialize_f690.asm
;   Microcontroller:    PIC16F690
;   Assembled using:    MPLAB IDE v7.20
;
;   Author:             Steven Bible
;   Company:            Microchip Technology Inc.
;
;----------------------------------------------------------------------
;
;   Program Description:
;
;   Initialize the PIC16F690 to known initial values.
;
;----------------------------------------------------------------------
;
;   Revision History:
;
;   Ver. 0.1 - 2005-08-29: Initial write
;
;----------------------------------------------------------------------

    errorlevel  -302                ; suppress message 302 from list file

;----------------------------------------------------------------------
; Include Files
;----------------------------------------------------------------------

#include    "main.inc"              ; Project defines

;----------------------------------------------------------------------
; Global Defines
;----------------------------------------------------------------------

    ; Place the following #define directives in the main.inc file

;#define PORT_A_TRIS  b'<value>'
;#define PORT_B_TRIS  b'<value>'
;#define PORT_C_TRIS  b'<value>'

;----------------------------------------------------------------------
; Local Defines
;----------------------------------------------------------------------

; None

;----------------------------------------------------------------------
; Variables
;----------------------------------------------------------------------

;--------------------
; Uninitialized Data Section
;
; The RES directive reserves space for data storage.

;        udata

; none

;--------------------
; Shared Uninitialized Data Section
;
; Variables that are allocated in RAM that ais shared across all RAM banks (unbanked RAM). 
; The RES directive reserves space for data storage.

;        udata_shr

; none

;--------------------
; Global Variables
;
; Variables defined in this module that are made available to other 
; modules. The <label> must be defined prior to using the GLOBAL
; directive in this module. 

;    global  <label>


;----------------------------------------------------------------------
; Begin Executable Code Segment
;----------------------------------------------------------------------

;[<label>]  code    [<ROM_Address>]

        code

;----------------------------------------------------------------------
; Subroutine: Initialize PIC16F690
;   
; Description: 
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: 
;   
; Output: 
;   
;----------------------------------------------------------------------

init_f690
    global  init_f690

;----------------------------------------
; Clock Sources (Section 3.0)
;
; The PIC16F690 can be configured in one of eight oscillator modes via
; the FOSC bits in the Configuration Word (Section 15). Refer to the 
; Configuration Bits section above for oscillator modes.
;
; Internal Clock Modes (Section 3.3.4)
;
; The PIC16F690 has two independant internal oscillators:
;   - a calibrated HFINTOSC (8 MHz), and
;   - an uncalibrated LFINTRC (31 kHz)
; The 8 MHz HFINTOSC drives a postscaler and multiplexer which can 
; provide one of six clock frequencies:
;
;   IRCF2  IRCF1  IRCF0
;     0      0      0    =   31 kHz
;     0      0      1    =  125 kHz
;     0      1      0    =  250 kHz
;     0      1      1    =  500 kHz
;     1      0      0    =    1 MHz
;     1      0      1    =    2 MHz
;     1      1      0    =    4 MHz
;     1      1      1    =    8 MHz

        banksel OSCCON              ; Bank 1

        ; IRCF2:IRCF0 = 111 -> 8 Mhz
        bsf     OSCCON, IRCF2       ; Internal Oscillator Frequency Select bit 2
        bsf     OSCCON, IRCF1       ; Internal Oscillator Frequency Select bit 1
        bsf     OSCCON, IRCF0       ; Internal Oscillator Frequency Select bit 0

;----------------------------------------
; Oscillator Tuning (OSCTUNE) Register (Section 3.4.2.1)
;
; The 8 MHz INTOSC's has been calibrated at the factory. It can be
; adjusted by writing to the OSCTUNE register. The OSCTUNE register
; is 5 bits long and has a tuning range of plus and minus 12 percent.

;                 xxx43210
        movlw   B'00000000'         ; Set OSCTUNE to center frequency
        movwf   OSCTUNE

;----------------------------------------
; Clock Sources and Oscillator Switching (Section 3.5)
;
; The PIC16F690 can switch the system clock source between the external
; and internal oscillators via the System Clock Select (SCS) bit.

        bcf     OSCCON, SCS         ; system clock source comes from the
                                    ; oscillator mode configured by the FOSC bits

;----------------------------------------
; I/O Ports (Section 4)
;
; Clear I/O Ports

        banksel PORTA               ; Bank 0

        clrf    PORTA
        clrf    PORTB
        clrf    PORTC

; Store PORT_A_TRIS, PORT_B_TRIS and PORT_C_TRIS values defined in main.inc
; into the TRISA, TRISB and TRISC direction registers

        banksel TRISA               ; Bank 1

        movlw   PORT_A_TRIS
        movwf   TRISA

        movlw   PORT_B_TRIS
        movwf   TRISB

        movlw   PORT_C_TRIS
        movwf   TRISC

;----------------------------------------
; PORTA Weak Pull-up Register (WPUA) (Section 4.2.1)
;
; Each of the PORTA pins, except RA3, has an individually configurable
; weak internal pull-up. Control bits WPUAx enable or disable each 
; pull-up. Refer to Register 4-3. Each weak pull-up is automatically
; turned off when the port pin is configured as an output. The pull-ups
; are disabled on a Power-on Reset by the /RAPU bit (see OPTION Register
; below).

        banksel WPUA                ; Bank 1

;    PORTA Pins = xx54x210
        movlw   B'00000000'         ;  no pull-ups enabled
        movwf   WPUA

;----------------------------------------
; PORTB Weak Pull-up Register (WPUB) (Section 4.4.1)
;
; Each of the PORTB pins has an individually configurable weak internal
; pull-up. Control bits WPUB<7:4> enable or disable each pull-up. Refer
; to Register 4-8. Each weak pull-up is automatically turned off when 
; the port pin is configured as an output. The pull-ups are disabled on
; a Power-on Reset by the /RAPU bit (see OPTION Register).

        banksel WPUB                ; Bank 2

;    PORTB Pins = 7654xxxx
        movlw   B'00000000'         ; no pull-ups enabled
        movwf   WPUB

;----------------------------------------
; PORTA Interrupt-on-Change Register (IOCA) (Section 4.2.2)
;
; Each of the PORTA pins is individually configurable as an interrupt-
; on-change pin. Control bits IOCAx enable or disable the interrupt 
; function for each pin. Refer to Register 4-4. The interrupt-on-change
; is disabled on a Power-on Reset.
;
; Note: Global, port change, and peripheral interrupt enables 
;       (GIE, RAIE and PEIE) must be enabled for individual interrupts
;       to be recognized (see INTCON register below).

        banksel IOCA                ; Bank 1

;    PORTA Pins = xx54x210
        movlw   B'00000000'         ; PORTA interrupt-on-change disable
        movwf   IOCA

;----------------------------------------
; PORTB Interrupt-on-Change Register (IOCB) (Section 4.4.2)
;
; Four of the PORTB pins are individually configurable as an interrupt-
; on-change pin. Control bits IOCB<7:4> enable or disable the interrupt 
; function for each pin. Refer to Register 4-8. The interrupt-on-change
; is disabled on a Power-on Reset.
;
; Note: Global, port change, and peripheral interrupt enables 
;       (GIE, RAIE and PEIE) must be enabled for individual interrupts
;       to be recognized (see INTCON register below).

        banksel IOCB                ; Bank 2

;    PORTB Pins = 7654xxxx
        movlw   B'00000010'         ; PORTB interrupt-on-change disabled
        movwf   IOCB

;----------------------------------------
; OPTION Register (OPTION_REG) (Section 2.2.2.2)
;
; The OPTION_REG contains control bits to configure:
;   Weak pull-ups on PORTA (see also WPUA Register above)
;   External RA2/INT interrupt
;   TMR0
;   TMR0/WDT prescaler

        banksel OPTION_REG          ; Bank 1

        bsf     OPTION_REG, NOT_RABPU ; PORTA/PORTB pull-ups: disabled

        bsf     OPTION_REG, INTEDG  ; Interrupt Edge: on rising edge of RA2/INT pin

; TIMER0 Module (Section 5.0)
;
; The Timer0 module timer/counter has the following features:
;   8-bit timer/counter
;   Readable and writable
;   8-bit software programmable prescaler
;   Internal or external clock select
;   Interrupt on overflow from 0xFF to 0x00
;   Edge select for external clock

        bcf     OPTION_REG, T0CS    ; TMR0 Clock Source: internal instruction cycle (CLKOUT)
        bcf     OPTION_REG, T0SE    ; TMR0 Source Edge: increment low-to-high transition on GP2/T0CKI pin

        bcf     OPTION_REG, PSA     ; Prescaler Assignment: assigned to TIMER0

        ; TMR0 Prescaler Rate: 1:2
        bcf     OPTION_REG, PS2     ; Prescaler Rate Select bit 2
        bcf     OPTION_REG, PS1     ; Prescaler Rate Select bit 1
        bcf     OPTION_REG, PS0     ; Prescaler Rate Select bit 0

;----------------------------------------
; TIMER1 Module with Gate Control (Section 6)
;
; The TIMER1 Control Register (T1CON) is used to enable/disable TIMER1
; and select various features of the TIMER1 module.

        banksel T1CON               ; Bank 0

        bcf     T1CON, TMR1CS       ; TIMER1 Clock Source Select: Internal Clock (Fosc/4)

        ; If TMR1CS=0 this bit is ignored
        bsf     T1CON, NOT_T1SYNC   ; TIMER1 External Clock Input Sync Control: do not syncronize external clock input

        ; T1OSCEN only if INTOSC without CLKOUT oscillator is active, else ignored
        bcf     T1CON, T1OSCEN      ; LP Oscillator Enable Control: LP oscillator off

        ; TIMER1 Input Prescale Select: 1:1
        bcf     T1CON, T1CKPS1      ; TIMER1 Input Clock Prescale Select bit 1
        bcf     T1CON, T1CKPS0      ; TIMER1 Input Clock Prescale Select bit 0

        ; TMR1GE only if TMR1ON = 1, else ignored
        bcf     T1CON, TMR1GE       ; TIMER1 Gate Enable: Timer1 is on

        bcf     T1CON, T1GINV       ; TIMER1 gate is not inverted

        bcf     T1CON, TMR1ON       ; TIMER1: disabled

;----------------------------------------
; TIMER2 Module (Section 7)
;
; The Timer2 module timer has the following features:
;   - 8-bit timer (TMR2 register)
;   - 8-bit period register (PR2)
;   - Readable and writable (both registers)
;   - Software programmable prescaler (1:1, 1:4, 1:16)
;   - Software programmable postscaler (1:1 to 1:16)
;   - Interrupt on TMR2 match with PR2

        banksel T2CON               ; Bank 0

        ; Timer2 Output Postscale Select: 1:1 postscale
        bcf     T2CON, TOUTPS3
        bcf     T2CON, TOUTPS2
        bcf     T2CON, TOUTPS1
        bcf     T2CON, TOUTPS0

        ; Timer2 Clock Prescale Select: Prescaler is 1
        bcf     T2CON, T2CKPS1
        bcf     T2CON, T2CKPS0

        bcf     T2CON, TMR2ON       ; Timer2: Off

;----------------------------------------
; Comparator Module (Section 8)
; Voltage Reference Module (Section 9)
;
; The PIC16F690 devices have one analog comparator. The inputs to
; the comparator are multiplexed with the RA0 and RA1 pins. There is
; an on-chip Comparator Voltage Reference that can also be applied to
; an input of the comparator. In addition, RA2 can be configured as
; the comparator output. The Comparator Control Register (CMCON)
; contains bits to control the comparator. The Voltage Reference
; Control Register (VRCON) controls the voltage reference module.

        banksel CM1CON0             ; Bank 2

        bcf     CM1CON0, C1ON       ; C1 Comparator: disabled
        bcf     CM2CON0, C2ON       ; C2 Comparator: disabled

        ; VRCON (Register 9-1)

        bcf     VRCON, C1VREN       ; Comparator 1 Voltage Reference: 0.6V constant
        bcf     VRCON, C2VREN       ; Comparator 2 Voltage Reference: 0.6V constant

        bcf     VRCON, VP6EN        ; 0.6V Reference: disabled

;        bcf     VRCON, VRR          ; CVref Range Selection: High Range

;        bcf     VRCON, VR3          ; CVref value selection bit 3
;        bcf     VRCON, VR2          ; CVref value selection bit 2
;        bcf     VRCON, VR1          ; CVref value selection bit 1
;        bcf     VRCON, VR0          ; CVref value selection bit 0

;----------------------------------------
; Analog-to-Digital Converter (A/D) Module (Section 10.0)
;
; The analog-to-digital converter (A/D) allows conversion of an analog
; input signal to a 10-bit binary representation of that signal. The
; PIC16F690 has twelve analog inputs, plus two internal inputs, 
; multiplexed into one sample and hold circuit. 
;
; There are four registers to control the functions of the A/D module:
;   A/D Control Register 0 (ADCON0)
;   A/D Control Register 1 (ADCON1)
;   Analog Select Register (ANSEL)
;   Analog Select High Register (ANSELH)
;
; Note: When using PORT pins as analog inputs, ensure the TRISA, TRISB
; or TRISC register bits are set (= 1) for input.

        banksel ADCON0              ; Bank 0

        bcf     ADCON0, ADFM        ; A/D Result Formed: left justified
        bcf     ADCON0, VCFG        ; Voltage Reference: Vdd

        bcf     ADCON0, ADON        ; ADC is shut-off and consumes no operating current

        banksel ADCON1              ; Bank 1

        ; select A/D Conversion Clock Source: Fosc/16
        bsf     ADCON1, ADCS2       ; A/D Conversion Clock Select bit 2
        bcf     ADCON1, ADCS1       ; A/D Conversion Clock Select bit 1
        bsf     ADCON1, ADCS0       ; A/D Conversion Clock Select bit 0

        banksel ANSELH              ; Bank 2

        ; select PORT pins that will be analog inputs: 
        bcf     ANSELH, ANS11       ; Analog Select RB5/AN11: digital I/O
        bcf     ANSELH, ANS10       ; Analog Select RB4/AN10: digital I/O
        bcf     ANSELH, ANS9        ; Analog Select RC7/AN9: digital I/O
        bcf     ANSELH, ANS8        ; Analog Select RC6/AN8: digital I/O

        bcf     ANSEL, ANS7         ; Analog Select RC3/AN7: digital I/O
        bcf     ANSEL, ANS6         ; Analog Select RC2/AN6: digital I/O
        bcf     ANSEL, ANS5         ; Analog Select RC1/AN5: digital I/O
        bcf     ANSEL, ANS4         ; Analog Select RC0/AN4: digital I/O
        bcf     ANSEL, ANS3         ; Analog Select RA3/AN3: digital I/O
        bcf     ANSEL, ANS2         ; Analog Select RA2/AN2: digital I/O
        bcf     ANSEL, ANS1         ; Analog Select RA1/AN1/Vref: digital I/O
        bcf     ANSEL, ANS0         ; Analog Select RA0/AN0: digital I/O

;----------------------------------------
; Enhanced Capture/Compare/PWM+ (ECCP+) Module (Section 12)
;
; The enhanced Capture/Compare/PWM+ (ECCP+) module contains a 16-bit
; register which can operate as a:
;   - 16-bit Capture register
;   - 16-bit Compare register
;   - PWM Master/Slave Duty Cycle register
;
; The CCP1CON register controls the operation of ECCP+.

        banksel CCP1CON             ; Bank 0

        ; ECCP Mode Select: CCP off (resets ECCP module)
        bcf     CCP1CON, CCP1M3
        bcf     CCP1CON, CCP1M2
        bcf     CCP1CON, CCP1M1
        bcf     CCP1CON, CCP1M0

;----------------------------------------
; Enhanced Universal Asynchronous Receiver Transmitter (EUSART) (Section 13)
;
; The USART can be configured in the following modes:
;   - Asynchronous (full-duplex) with:
;       - Auto wake-up on character reception
;       - Auto baud calibration
;       - 12-bit Break character transmission
;   - Synchronous - Master (half-duplex) with selectable clock polarity
;   - Synchronous - Slave (half-duplex) with selectable clock polarity
;
; To configure pins RB5/AN11/RX/DT and RB7/TX/CK to the USART set bits:
;   - SPEN (RCSTA<7>) = 1
;   - TRISB<5> = 1
;   - TRISB<7> = 1
;   NOTE: The USART control will automatically reconfigure the pin from
;         input to output as needed.

        ; hold the USART in reset until configuration is complete

        banksel RCSTA               ; Bank 0

        bcf     RCSTA, SPEN         ; Serial Port Enable bit: serial port disabled

;----------------------------------------
; Synchronous Serial Port Module (Section 13)
;
; The Synchronous Serial Port (SSP) module is a serial interface used to
; communicate with other peripheral or microcontroller devices. These 
; peripheral devices may be serial EEPROMs, shift registers, display
; drivers, A/D converters, etc. The SSP module can operate in one of two
; modes:
;   - Serial Peripheral Interface (SPI)
;   - Inter-Integrated Circuit (I2C)


        banksel SSPSTAT
        clrf    SSPSTAT

        banksel SSPCON
        bcf     SSPCON, SSPEN       ; Synchronous Serial Port Enable: Disabled


;----------------------------------------

        return

;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
